import abc
import copy
import operator
from abjad.tools import durationtools
from abjad.tools import formattools
from abjad.tools import sequencetools
from abjad.tools.componenttools.Component import Component


class Leaf(Component):

    ### CLASS ATTRIBUTES ##

    __metaclass__ = abc.ABCMeta

    # TODO: see if _grace and _after_grace can be removed #
    __slots__ = ('_after_grace', '_grace', '_leaf_index',
        '_duration_multiplier', '_written_duration',
        '_written_pitch_indication_is_nonsemantic',
        '_written_pitch_indication_is_at_sounding_pitch',
        'after_grace', 'grace', )

    ### INITIALIZER ###

    def __init__(self, written_duration, duration_multiplier=None):
        Component.__init__(self)
        self._duration_multiplier = duration_multiplier
        self._leaf_index = None
        self.written_duration = durationtools.Duration(written_duration)
        self.written_pitch_indication_is_nonsemantic = False
        self.written_pitch_indication_is_at_sounding_pitch = True

    ### SPECIAL METHODS ###

    def __and__(self, arg):
        return self._operate(arg, operator.__and__)

    def __copy__(self, *args):
        return self._copy_with_marks_but_without_children_or_spanners()

    # necessary to make sure deepcopied leaves copy grace containers correctly
    __deepcopy__ = __copy__

    def __getnewargs__(self):
        result = []
        result.append(self.written_duration)
        if self.duration_multiplier is not None:
            result.append(self.duration_multiplier)
        return tuple(result)

    def __or__(self, arg):
        return self._operate(arg, operator.__or__)

    def __repr__(self):
        return '%s(%r)' % (type(self).__name__, self._compact_representation)

    def __str__(self):
        return self._compact_representation

    def __sub__(self, arg):
        return self._operate(arg, operator.__sub__)

    def __xor__(self, arg):
        return self._operate(arg, operator.__xor__)

    ### PRIVATE PROPERTIES ###

    @property
    def _compact_representation(self):
        return '({})'.format(self._formatted_duration)

    @property
    def _format_pieces(self):
        return self.lilypond_format.split('\n')

    @property
    def _formatted_duration(self):
        duration_string = self.written_duration.lilypond_duration_string
        if self.duration_multiplier is not None:
            return '%s * %s' % (duration_string, self.duration_multiplier)
        else:
            return duration_string

    ### PRIVATE METHODS ###

    def _copy_override_and_set_from_leaf(self, leaf):
        if getattr(leaf, '_override', None) is not None:
            self._override = copy.copy(leaf.override)
        if getattr(leaf, '_set', None) is not None:
            self._set = copy.copy(leaf.set)

    def _copy_with_marks_but_without_children_or_spanners(self):
        from abjad.tools import gracetools
        new = Component._copy_with_marks_but_without_children_or_spanners(self)
        for grace_container in gracetools.get_grace_containers_attached_to_leaf(self):
            new_grace_container = copy.deepcopy(grace_container)
            new_grace_container(new)
        return new
        
    def _format_after_slot(leaf, format_contributions):
        result = []
        result.append(('spanners', format_contributions.get('after', {}).get('spanners', [])))
        result.append(('context marks', format_contributions.get('after', {}).get('context marks', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('after', {}).get('lilypond command marks', [])))
        result.append(('comments', format_contributions.get('after', {}).get('comments', [])))
        return result

    def _format_agrace_body(leaf):
        result = []
        if hasattr(leaf, '_after_grace'):
            after_grace = leaf.after_grace
            if len(after_grace):
                result.append(after_grace.lilypond_format)
        return ['agrace body', result]

    def _format_agrace_opening(leaf):
        result = []
        if hasattr(leaf, '_after_grace'):
            if len(leaf.after_grace):
                result.append(r'\afterGrace')
        return ['agrace opening', result]

    def _format_before_slot(leaf, format_contributions):
        result = []
        result.append(leaf._format_grace_body())
        result.append(('comments', format_contributions.get('before', {}).get('comments', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('before', {}).get('lilypond command marks', [])))
        result.append(('context marks', format_contributions.get('before', {}).get('context marks', [])))
        result.append(('grob overrides', format_contributions.get('grob overrides', [])))
        result.append(('context settings', format_contributions.get('context settings', [])))
        result.append(('spanners', format_contributions.get('before', {}).get('spanners', [])))
        return result

    def _format_close_brackets_slot(leaf, format_contributions):
        return []

    def _format_closing_slot(leaf, format_contributions):
        result = []
        result.append(leaf._format_agrace_body())
        result.append(('spanners', format_contributions.get('closing', {}).get('spanners', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('closing', {}).get('lilypond command marks', [])))
        result.append(('context marks', format_contributions.get('closing', {}).get('context marks', [])))
        result.append(('comments', format_contributions.get('closing', {}).get('comments', [])))
        return result

    def _format_contents_slot(leaf, format_contributions):
        result = []
        result.append(leaf._format_leaf_body(format_contributions))
        return result


    def _format_grace_body(leaf):
        result = []
        if hasattr(leaf, '_grace'):
            grace = leaf.grace
            if len(grace):
                result.append(grace.lilypond_format)
        return ['grace body', result]

    def _format_leaf_body(leaf, format_contributions):
        result = leaf._format_leaf_nucleus()[1]
        right = format_contributions.get('right', {})
        if right:
            result.extend(right.get('stem tremolos', []))
            result.extend(right.get('articulations', []))
            result.extend(right.get('lilypond command marks', []))
            result.extend(right.get('context marks', []))
            result.extend(right.get('spanners', []))
            result.extend(right.get('comments', []))
        result = [' '.join(result)]
        markup = right.get('markup')
        if markup:
            if len(markup) == 1:
                result[0] += ' {}'.format(markup[0])
            else:
                result.extend('\t{}'.format(x) for x in markup)
        return ['leaf body', result]

    # TODO: subclass this properly for chord
    def _format_leaf_nucleus(leaf):
        from abjad.tools.chordtools.Chord import Chord
        if not isinstance(leaf, Chord):
            return ['nucleus', leaf._body]
        result =  []
        chord = leaf
        note_heads = chord.note_heads
        if any(['\n' in x.lilypond_format for x in note_heads]):
            for note_head in note_heads:
                format = note_head.lilypond_format
                format_list = format.split('\n')
                format_list = ['\t' + x for x in format_list]
                result.extend(format_list)
            result.insert(0, '<')
            result.append('>')
            result = '\n'.join(result)
            result += str(chord._formatted_duration)
        else:
            result.extend([x.lilypond_format for x in note_heads])
            result = '<%s>%s' % (' '.join(result), chord._formatted_duration)
        # single string, but wrapped in list bc contribution
        return ['nucleus', [result]]

    def _format_open_brackets_slot(leaf, format_contributions):
        return []

    def _format_opening_slot(leaf, format_contributions):
        result = []
        result.append(('comments', format_contributions.get('opening', {}).get('comments', [])))
        result.append(('context marks', format_contributions.get('opening', {}).get('context marks', [])))
        result.append(('lilypond command marks', 
            format_contributions.get('opening', {}).get('lilypond command marks', [])))
        result.append(('spanners', format_contributions.get('opening', {}).get('spanners', [])))
        result.append(leaf._format_agrace_opening())
        return result

    def _operate(self, arg, operator):
        assert isinstance(arg, Leaf)
        from abjad.tools import leaftools
        from abjad.tools import pitchtools
        self_pairs = set(pitchtools.list_named_chromatic_pitches_in_expr(self))
        arg_pairs = set(pitchtools.list_named_chromatic_pitches_in_expr(arg))
        pairs = operator(self_pairs, arg_pairs)
        if len(pairs) == 0:
            pairs = [None]
        elif len(pairs) == 1:
            pairs = list(pairs)
        else:
            pairs = [tuple(pairs)]
        leaves = leaftools.make_leaves(pairs, self.written_duration)
        leaf = leaves[0]
        return leaf

    def _process_contribution_packet(self, contribution_packet):
        result = ''
        for contributor, contributions in contribution_packet:
            if contributions:
                if isinstance(contributor, tuple):
                    contributor = '\t' + contributor[0] + ':\n'
                else:
                    contributor = '\t' + contributor + ':\n'
                result += contributor
                for contribution in contributions:
                    contribution = '\t\t' + contribution + '\n'
                    result += contribution
        return result

    def _report_format_contributors(self):
        format_contributions = formattools.get_all_format_contributions(self)
        report = ''
        report += 'slot 1:\n'
        report += self._process_contribution_packet(self._format_before_slot(format_contributions))
        report += 'slot 3:\n'
        report += self._process_contribution_packet(self._format_opening_slot(format_contributions))
        report += 'slot 4:\n'
        report += '\tleaf body:\n'
        report += '\t\t' + self._format_contents_slot(format_contributions)[0][1][0] + '\n'
        report += 'slot 5:\n'
        report += self._process_contribution_packet(self._format_closing_slot(format_contributions))
        report += 'slot 7:\n'
        report += self._process_contribution_packet(self._format_after_slot(format_contributions))
        return report

    ### PUBLIC PROPERTIES ###

    @property
    def duration_in_seconds(self):
        from abjad.tools import contexttools
        tempo = contexttools.get_effective_tempo(self)
        if tempo is not None and not tempo.is_imprecise:
            result = self.prolated_duration / tempo.duration / tempo.units_per_minute * 60
            return durationtools.Duration(result)
        raise MissingTempoError

    @apply
    def duration_multiplier():
        def fget(self):
            return self._duration_multiplier
        def fset(self, expr):
            if expr is None:
                self._duration_multiplier = None
            else:
                duration_multiplier = durationtools.Multiplier(expr)
                assert 0 <= duration_multiplier
                self._duration_multiplier = duration_multiplier
        return property(**locals())

    @property
    def leaf_index(self):
        self._update_prolated_offset_values_of_entire_score_tree_if_necessary()
        return self._leaf_index

    @property
    def multiplied_duration(self):
        if self.written_duration:
            if self.duration_multiplier is not None:
                return self.written_duration * self.duration_multiplier
            else:
                return durationtools.Duration(self.written_duration)
        else:
            return None

    @property
    def preprolated_duration(self):
        return self.multiplied_duration

    @apply
    def written_duration():
        def fget(self):
            return self._written_duration
        def fset(self, expr):
            rational = durationtools.Duration(expr)
            if not rational.is_assignable:
                raise AssignabilityError('not assignable duration: "%s".' % str(rational))
            self._written_duration = rational
        return property(**locals())

    @apply
    def written_pitch_indication_is_at_sounding_pitch():
        def fset(self, arg):
            '''Read / write flag to be set to false when pitch indication is transposed.
            '''
            if not isinstance(arg, bool):
                raise TypeError
            self._written_pitch_indication_is_at_sounding_pitch = arg
        def fget(self):
            return self._written_pitch_indication_is_at_sounding_pitch
        return property(**locals())

    @apply
    def written_pitch_indication_is_nonsemantic():
        def fset(self, arg):
            '''Read / write flag to be set when using leaves only graphically.

            setting this value to true sets sounding pitch indicator to false.
            '''
            if not isinstance(arg, bool):
                raise TypeError
            self._written_pitch_indication_is_nonsemantic = arg
            if arg == True:
                self.written_pitch_indication_is_at_sounding_pitch = False
        def fget(self):
            return self._written_pitch_indication_is_nonsemantic
        return property(**locals())



def get_time_signature_marks_attached_to_component(component):
    r'''.. versionadded:: 2.3

    Get time signature marks attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> contexttools.TimeSignatureMark((2, 4))(staff)
        TimeSignatureMark((2, 4))(Staff{4})

    ::

        >>> f(staff)
        \new Staff {
            \time 2/4
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> contexttools.get_time_signature_marks_attached_to_component(staff)
        (TimeSignatureMark((2, 4))(Staff{4}),)

    Return tuple of zero or more time_signature marks.
    '''
    from abjad.tools import contexttools

    return contexttools.get_context_marks_attached_to_component(
        component, klasses=(contexttools.TimeSignatureMark,))



from abjad.tools import contexttools
from abjad.tools import schemetools
from abjad.tools import scoretools
from abjad.tools import stafftools


def make_monophonic_percussion_score_from_nonoverlapping_intervals(intervals, colorkey=None):
    '''Create a monophonic percussion score from nonoverlapping interval collection `intervals`.

    Return Score.
    '''
    from abjad.tools import timeintervaltools

    voice = timeintervaltools.make_voice_from_nonoverlapping_intervals(intervals, colorkey)

    staff = stafftools.Staff(voice[:])
    staff.override.staff_symbol.line_count = 1
    contexttools.ClefMark('percussion')(staff)

    score = scoretools.Score([staff])
    score.override.glissando.thickness = 5
    score.override.note_head.style = 'harmonic'
    score.override.rest.transparent = True
    score.override.spacing_spanner.strict_note_spacing = True
    score.override.glissando.breakable = True

    padding = 0.5
    bound_details = schemetools.Scheme(
        ('right', 
            schemetools.SchemePair('attach-dir', 0),
            schemetools.SchemePair('padding', padding)),
        ('left',
            schemetools.SchemePair('attach-dir', 0),
            schemetools.SchemePair('padding', padding)),
        quoting="'")
    score.override.glissando.bound_details = bound_details

    return score



from abjad.tools import stringtools
from abjad.tools.spannertools.DirectedSpanner.DirectedSpanner import DirectedSpanner


class TieSpanner(DirectedSpanner):
    r'''Abjad tie spanner::

        >>> staff = Staff(notetools.make_repeated_notes(4))
        >>> tietools.TieSpanner(staff[:])
        TieSpanner(c'8, c'8, c'8, c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8 ~
            c'8 ~
            c'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return tie spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, music=None, direction=None):
        DirectedSpanner.__init__(self, music, direction)

    ### PRIVATE METHODS ###
    
    def _copy_keyword_args(self, new):
        DirectedSpanner._copy_keyword_args(self, new)

    def _format_right_of_leaf(self, leaf):
        result = []
        if not self._is_my_last_leaf(leaf):
            if self.direction is not None:
                result.append('{} ~'.format(stringtools.arg_to_tridirectional_lilypond_symbol(self.direction)))
            else:
                result.append('~')
        return result



def uppercamelcase_to_underscore_delimited_lowercase(string):
    r'''.. versionadded:: 2.6

    Change uppercamelcase `string` to underscore-delimited lowercase::

        >>> string = 'KeySignatureMark'

    ::

        >>> stringtools.uppercamelcase_to_underscore_delimited_lowercase(string)
        'key_signature_mark'

    Return string.

    .. versionchanged:: 2.9
        renamed ``iotools.uppercamelcase_to_underscore_delimited_lowercase()`` to
        ``stringtools.uppercamelcase_to_underscore_delimited_lowercase()``.
    '''

    words = []
    current_word = string[0].lower()
    for letter in string[1:]:
        if letter.isupper():
            words.append(current_word)
            current_word = letter.lower()
        else:
            current_word = current_word + letter
    words.append(current_word)
    result = '_'.join(words)
    return result



from abjad.tools import componenttools
from abjad.tools import spannertools


def are_components_in_same_tie_spanner(components):
    r'''True when `components` are in same tie spanner::

        >>> voice = Voice("c'8 ~ c' d' ~ d'")

    ::

        >>> f(voice)
        \new Voice {
            c'8 ~
            c'8
            d'8 ~
            d'8
        }

    ::

        >>> tietools.are_components_in_same_tie_spanner(voice[:2])
        True

    Otherwise false::

        >>> tietools.are_components_in_same_tie_spanner(voice[1:3])
        False

    Return boolean.

    .. versionchanged:: 2.0
        renamed ``tietools.are_in_same_spanner()`` to
        ``tietools.are_components_in_same_tie_spanner()``.
    '''
    from abjad.tools import tietools

    assert componenttools.all_are_components(components)

    try:
        first = components[0]
        try:
            first_tie_spanner = spannertools.get_the_only_spanner_attached_to_component(
                first, tietools.TieSpanner)
            for component in components[1:]:
                component_tie_spanner = spannertools.get_the_only_spanner_attached_to_component(
                    component, tietools.TieSpanner)
                if component_tie_spanner is not first_tie_spanner:
                    return False
        except MissingSpannerError:
            return False
    except IndexError:
        return True

    return True



# -*- encoding: utf-8 -*-
from abjad import *


def configure_lilypond_file(lilypond_file):

    lilypond_file.global_staff_size = 8

    context_block = lilypondfiletools.ContextBlock()
    context_block.context_name = r'Staff \RemoveEmptyStaves'
    context_block.override.vertical_axis_group.remove_first = True
    lilypond_file.layout_block.context_blocks.append(context_block)

    lilypond_file.paper_block.system_separator_markup = marktools.LilyPondCommandMark('slashSeparator')
    lilypond_file.paper_block.bottom_margin = lilypondfiletools.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.top_margin =    lilypondfiletools.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.left_margin =   lilypondfiletools.LilyPondDimension(0.75, 'in')
    lilypond_file.paper_block.right_margin =  lilypondfiletools.LilyPondDimension(0.5, 'in')
    lilypond_file.paper_block.paper_width =   lilypondfiletools.LilyPondDimension(5.25, 'in')
    lilypond_file.paper_block.paper_height =  lilypondfiletools.LilyPondDimension(7.25, 'in')

    lilypond_file.header_block.composer = markuptools.Markup('Arvo Pärt')
    lilypond_file.header_block.title = markuptools.Markup('Cantus in Memory of Benjamin Britten (1980)')




from abjad import *


#def test_containertools_replace_container_slice_with_rests_01():
#
#    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8")
#    containertools.replace_container_slice_with_rests(staff, stop=5, decrease_durations_monotonically=True)
#
#    r'''
#    \new Staff {
#        r2
#        r8
#        a'8
#    }
#    '''
#
#    assert staff.lilypond_format == "\\new Staff {\n\tr2\n\tr8\n\ta'8\n}"
#
#
#def test_containertools_replace_container_slice_with_rests_02():
#
#    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8")
#    containertools.replace_container_slice_with_rests(staff, start=-5, decrease_durations_monotonically=True)
#
#    r'''
#    \new Staff {
#        c'8
#        r2
#        r8
#    }
#    '''
#
#    assert staff.lilypond_format == "\\new Staff {\n\tc'8\n\tr2\n\tr8\n}"


def test_containertools_replace_container_slice_with_rests_03():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8")
    containertools.replace_container_slice_with_rests(staff, stop=5, decrease_durations_monotonically=False)

    r'''
    \new Staff {
        r8
        r2
        a'8
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\tr8\n\tr2\n\ta'8\n}"


def test_containertools_replace_container_slice_with_rests_04():

    staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8")
    containertools.replace_container_slice_with_rests(staff, start=-5, decrease_durations_monotonically=False)

    r'''
    \new Staff {
        c'8
        r8
        r2
    }
    '''

    assert staff.lilypond_format == "\\new Staff {\n\tc'8\n\tr8\n\tr2\n}"



import abc
import copy
import types
from abjad.tools import durationtools
from abjad.tools import leaftools
from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.rhythmmakertools.RhythmMaker import RhythmMaker


class IncisedRhythmMaker(RhythmMaker):
    '''.. versionadded 2.8

    Abstract base class for rhythm-makers that incise some or 
    all of the output cells they produce.

    Rhythm makers can incise the edge of every output cell.
    
    Or rhythm-makers can incise only the start of the first output cell 
    and the end of the last output cell.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    _default_positional_input_arguments = (
        [8],
        [1, 2, 3, 4],
        [1],
        [1],
        32,
        )

    ### INITIALIZER ###

    def __init__(self, prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, talea_denominator,
        prolation_addenda=None, secondary_divisions=None,
        prefix_talea_helper=None, prefix_lengths_helper=None,
        suffix_talea_helper=None, suffix_lengths_helper=None,
        prolation_addenda_helper=None, secondary_divisions_helper=None,
        decrease_durations_monotonically=True, tie_rests=False,
        beam_each_cell=False, beam_cells_together=False
        ):
        RhythmMaker.__init__(self,
            beam_each_cell=beam_each_cell,
            beam_cells_together=beam_cells_together)
        prolation_addenda = self._none_to_new_list(prolation_addenda)
        secondary_divisions = self._none_to_new_list(secondary_divisions)
        prefix_talea_helper = self._none_to_trivial_helper(prefix_talea_helper)
        prefix_lengths_helper = self._none_to_trivial_helper(prefix_lengths_helper)
        suffix_talea_helper = self._none_to_trivial_helper(suffix_talea_helper)
        suffix_lengths_helper = self._none_to_trivial_helper(suffix_lengths_helper)
        prolation_addenda_helper = self._none_to_trivial_helper(prolation_addenda_helper)
        secondary_divisions_helper = self._none_to_trivial_helper(secondary_divisions_helper)
        assert sequencetools.all_are_integer_equivalent_numbers(prefix_talea)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(prefix_lengths)
        assert sequencetools.all_are_integer_equivalent_numbers(suffix_talea)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(suffix_lengths)
        assert mathtools.is_positive_integer_equivalent_number(talea_denominator)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(prolation_addenda)
        assert sequencetools.all_are_nonnegative_integer_equivalent_numbers(secondary_divisions)
        assert isinstance(prefix_talea_helper, (types.FunctionType, types.MethodType))
        assert isinstance(prefix_lengths_helper, (types.FunctionType, types.MethodType))
        assert isinstance(suffix_talea_helper, (types.FunctionType, types.MethodType))
        assert isinstance(suffix_lengths_helper, (types.FunctionType, types.MethodType))
        assert isinstance(prolation_addenda_helper, (types.FunctionType, types.MethodType))
        assert isinstance(secondary_divisions_helper, (types.FunctionType, types.MethodType))
        assert isinstance(decrease_durations_monotonically, bool)
        assert isinstance(tie_rests, bool)
        self.prefix_talea = prefix_talea
        self.prefix_lengths = prefix_lengths
        self.suffix_talea = suffix_talea
        self.suffix_lengths = suffix_lengths
        self.prolation_addenda = prolation_addenda
        self.talea_denominator = talea_denominator
        self.secondary_divisions = secondary_divisions
        self.prefix_talea_helper = self._none_to_trivial_helper(prefix_talea_helper)
        self.prefix_lengths_helper = self._none_to_trivial_helper(prefix_lengths_helper)
        self.suffix_talea_helper = self._none_to_trivial_helper(suffix_talea_helper)
        self.suffix_lengths_helper = self._none_to_trivial_helper(suffix_lengths_helper)
        self.prolation_addenda_helper = self._none_to_trivial_helper(prolation_addenda_helper)
        self.secondary_divisions_helper = self._none_to_trivial_helper(secondary_divisions_helper)
        self.decrease_durations_monotonically = decrease_durations_monotonically
        self.tie_rests = tie_rests

    ### SPECIAL METHODS ###

    def __call__(self, divisions, seeds=None):
        duration_pairs, seeds = RhythmMaker.__call__(self, divisions, seeds)
        result = self._prepare_input(seeds)
        prefix_talea, prefix_lengths, suffix_talea, suffix_lengths = result[:-2]
        prolation_addenda, secondary_divisions = result[-2:]
        talee = (prefix_talea, suffix_talea, prolation_addenda, secondary_divisions)
        result = self._scale_talee(duration_pairs, self.talea_denominator, talee)
        duration_pairs, lcd, prefix_talea, suffix_talea = result[:-2]
        prolation_addenda, secondary_divisions = result[-2:]
        secondary_duration_pairs = self._make_secondary_duration_pairs(
            duration_pairs, secondary_divisions)
        numeric_map = self._make_numeric_map(secondary_duration_pairs,
            prefix_talea, prefix_lengths, suffix_talea, suffix_lengths, prolation_addenda)
        leaf_lists = self._numeric_map_and_talea_denominator_to_leaf_lists(numeric_map, lcd)
        if not self.prolation_addenda:
            return leaf_lists
        else:
            tuplets = self._make_tuplets(secondary_duration_pairs, leaf_lists)
            return tuplets

    def __eq__(self, other):
        return isinstance(other, type(self)) and all([
            self.prefix_talea == other.prefix_talea,
            self.prefix_lengths == other.prefix_lengths,
            self.suffix_talea == other.suffix_talea,
            self.suffix_lengths == other.suffix_lengths,
            self.prolation_addenda == other.prolation_addenda,
            self.talea_denominator == other.talea_denominator,
            self.secondary_divisions == other.secondary_divisions,
            ])

    ### PRIVATE METHODS ###

    @abc.abstractmethod
    def _make_middle_of_numeric_map_part(self, middle):
        pass

    def _make_numeric_map_part(self, numerator, prefix, suffix, is_note_filled=True):
        prefix_weight = mathtools.weight(prefix)
        suffix_weight = mathtools.weight(suffix)
        middle = numerator - prefix_weight - suffix_weight
        if numerator < prefix_weight:
            weights = [numerator]
            prefix = sequencetools.split_sequence_by_weights(prefix, weights, cyclic=False, overhang=False)[0]
        middle = self._make_middle_of_numeric_map_part(middle)
        suffix_space = numerator - prefix_weight
        if suffix_space <= 0:
            suffix = ()
        elif suffix_space < suffix_weight:
            weights = [suffix_space]
            suffix = sequencetools.split_sequence_by_weights(suffix, weights, cyclic=False, overhang=False)[0]
        numeric_map_part = prefix + middle + suffix
        return numeric_map_part

    def _numeric_map_and_talea_denominator_to_leaf_lists(self, numeric_map, lcd):
        leaf_lists = []
        for numeric_map_part in numeric_map:
            leaf_list = leaftools.make_leaves_from_talea(
                numeric_map_part, lcd,
                decrease_durations_monotonically=self.decrease_durations_monotonically, tie_rests=self.tie_rests)
            leaf_lists.append(leaf_list)
        return leaf_lists

    def _prepare_input(self, seeds):
        prefix_talea = self.prefix_talea_helper(self.prefix_talea, seeds)
        prefix_lengths = self.prefix_lengths_helper(self.prefix_lengths, seeds)
        suffix_talea = self.suffix_talea_helper(self.suffix_talea, seeds)
        suffix_lengths = self.suffix_lengths_helper(self.suffix_lengths, seeds)
        prolation_addenda = self.prolation_addenda_helper(self.prolation_addenda, seeds)
        secondary_divisions = self.secondary_divisions_helper(self.secondary_divisions, seeds)
        prefix_talea = sequencetools.CyclicTuple(prefix_talea)
        suffix_talea = sequencetools.CyclicTuple(suffix_talea)
        prefix_lengths = sequencetools.CyclicTuple(prefix_lengths)
        suffix_lengths = sequencetools.CyclicTuple(suffix_lengths)
        if prolation_addenda:
            prolation_addenda = sequencetools.CyclicTuple(prolation_addenda)
        else:
            prolation_addenda = sequencetools.CyclicTuple([0])
        secondary_divisions = sequencetools.CyclicTuple(secondary_divisions)
        return prefix_talea, prefix_lengths, \
            suffix_talea, suffix_lengths, prolation_addenda, secondary_divisions

    ### PUBLIC METHODS ###

    def reverse(self):
        r'''.. versionadded:: 2.10

        Reverse incised rhythm-maker.

        Return newly constructed rhythm-maker.
        '''
        new = copy.deepcopy(self)
        new.prefix_talea.reverse()
        new.prefix_lengths.reverse()
        new.suffix_talea.reverse()
        new.suffix_lengths.reverse()
        new.prolation_addenda.reverse()
        new.secondary_divisions.reverse()
        if new.decrease_durations_monotonically:
            new.decrease_durations_monotonically = False
        return new



from abjad.tools import durationtools


def scale_aggregate_duration_by_rational(intervals, rational):
    '''Scale the aggregate duration of all intervals in `intervals` by
    `rational`, maintaining the original start offset ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(-1, 3)
        >>> b = TimeInterval(6, 12)
        >>> c = TimeInterval(9, 16)
        >>> tree = TimeIntervalTree([a, b, c])
        >>> timeintervaltools.scale_aggregate_duration_by_rational(tree, Fraction(1, 3))
        TimeIntervalTree([
            TimeInterval(Offset(-1, 1), Offset(1, 3), {}),
            TimeInterval(Offset(4, 3), Offset(10, 3), {}),
            TimeInterval(Offset(7, 3), Offset(14, 3), {})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    rational = durationtools.Duration(rational)
    assert 0 < rational

    if not tree or rational == 1:
        return tree

    return timeintervaltools.TimeIntervalTree([
        x.shift_to_rational(
            ((x.start - tree.start) * rational) + tree.start).scale_by_rational(rational)\
            for x in tree
    ])



import os
import shutil
import py.test
from abjad.tools import *


@py.test.skip('Test runs incredibly slow.')
def test_AbjadBookProcessor___call___01():

    if __name__ == '__main__':
        directory = os.path.curdir
    else:
        directory = os.path.dirname(__file__)

    filepath = os.path.join(directory, 'text.rst.raw')

    with open(filepath, 'r') as f:
        lines = f.read().split('\n')

    book = abjadbooktools.AbjadBookProcessor(directory, lines, abjadbooktools.ReSTOutputFormat())

    result = book(directory)

    assert result == 'This is **paragraph 1**.\nNow comes some Abjad code\n\n::\n\n   >>> voice = Voice("c\'8 d\'8 e\'8")\n   >>> beamtools.BeamSpanner(voice)\n   BeamSpanner({c\'8, d\'8, e\'8})\n   >>> beam = _\n   >>> show(voice)\n\n.. image:: images/image-1.png\n\n::\n\n   >>> len(beam)\n   1\n   >>> show(voice)\n\n.. image:: images/image-2.png\n\n\nHere is **paragraph 2**, and more Abjad code.\nNotice that in the second block of abjad code I can reference objects and variables created in previous blocks:\n\n::\n\n   >>> spannertools.TrillSpanner(voice[4:])\n   TrillSpanner()\n   >>> f(voice)\n   \\new Voice {\n       c\'8 [\n       d\'8\n       e\'8 ]\n   }\n   >>> show(voice)\n\n.. image:: images/image-3.png\n\n\nHere is **paragraph 3**, and now a function definition.\nNote that this function definition can be used later.\nThe **strip_prompt=true** option tells abjad-book to print the code block as though it wasn\'t passed to the interpreter.\n\n::\n\n   def apply_articulations(components):\n       for i, component in enumerate(components):\n           if i % 2 == 0:\n               marktools.Articulation(\'.\')(component)\n           else:\n               marktools.Articulation(\'^\')(component)\n\n\nHere is **paragraph** 4, where we use the previous function to change the Voice we previously instantiated.\n\n::\n\n   >>> apply_articulations(voice)\n   >>> show(voice)\n\n.. image:: images/image-4.png\n\n\nAnd a final paragraph.\n'
    assert os.path.exists(os.path.join(directory, 'images'))
    shutil.rmtree(os.path.join(directory, 'images'))

    return result



import copy
from abjad.tools import containertools
from abjad.tools import sequencetools   
from abjad.tools import voicetools
from abjad.tools.quantizationtools.QTarget import QTarget


class BeatwiseQTarget(QTarget): 
    '''A beat-wise quantization target.

    Not composer-safe.

    Used internally by ``Quantizer``.

    Return ``BeatwiseQTarget`` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def beats(self):
        return tuple(self.items)
    
    @property
    def item_klass(self):
        from abjad.tools import quantizationtools
        return quantizationtools.QTargetBeat

    ### PRIVATE METHODS ###

    def _notate(self, grace_handler, attack_point_optimizer, attach_tempo_marks):
        voice = voicetools.Voice()

        # generate the first
        beat = self.items[0]
        components = beat.q_grid(beat.beatspan)
        if attach_tempo_marks:
            attachment_target = components[0]
            if isinstance(attachment_target, containertools.Container):
                attachment_target = attachment_target.leaves[0]
            copy.copy(beat.tempo)(attachment_target)
        voice.extend(components)

        # generate the rest pairwise, comparing tempi
        for beat_one, beat_two in sequencetools.iterate_sequence_pairwise_strict(self.items):
            components = beat_two.q_grid(beat_two.beatspan)
            if (beat_two.tempo != beat_one.tempo) and attach_tempo_marks:
                attachment_target = components[0]
                if isinstance(attachment_target, containertools.Container):
                    attachment_target = attachment_target.leaves[0]
                copy.copy(beat_two.tempo)(attachment_target)
            voice.extend(components)

        # apply tie chains, pitches, grace containers
        self._notate_leaves_pairwise(voice, grace_handler)

        # partition tie chains in voice
        attack_point_optimizer(voice)

        return voice



from abjad import *


def configure_score(score):
    score.set.proportional_notation_duration = schemetools.SchemeMoment(1, 56)
    score.set.tuplet_full_length = True
    score.override.bar_line.stencil = False
    score.override.bar_number.transparent = True
    score.override.spacing_spanner.uniform_stretching = True
    score.override.spacing_spanner.strict_note_spacing = True
    score.override.time_signature.stencil = False
    score.override.tuplet_bracket.padding = 2
    score.override.tuplet_bracket.staff_padding = 4
    score.override.tuplet_number.text = schemetools.Scheme('tuplet-number::calc-fraction-text')
    pass




def yield_groups_of_mixed_klasses_in_sequence(sequence, klasses):
    r'''.. versionadded:: 2.0

    Example 1. Yield groups of notes and chords at only the top level of score::

        >>> staff = Staff(r"\times 2/3 { c'8 d'8 r8 } \times 2/3 { r8 <e' g'>8 <f' a'>8 }")
        >>> staff.extend("g'8 a'8 r8 r8 <b' d''>8 <c'' e''>8")

    ::

        >>> f(staff)
        \new Staff {
            \times 2/3 {
                c'8
                d'8
                r8
            }
            \times 2/3 {
                r8
                <e' g'>8
                <f' a'>8
            }
            g'8
            a'8
            r8
            r8
            <b' d''>8
            <c'' e''>8
        }

    ::

        >>> for group in componenttools.yield_groups_of_mixed_klasses_in_sequence(
        ...     staff, (Note, Chord)):
        ...     group
        (Note("g'8"), Note("a'8"))
        (Chord("<b' d''>8"), Chord("<c'' e''>8"))

    Example 2. Yield groups of notes and chords at all levels of score::

        >>> leaves = iterationtools.iterate_leaves_in_expr(staff)

    ::

        >>> for group in componenttools.yield_groups_of_mixed_klasses_in_sequence(
        ...     leaves, (Note, Chord)):
        ...     group
        (Note("c'8"), Note("d'8"))
        (Chord("<e' g'>8"), Chord("<f' a'>8"), Note("g'8"), Note("a'8"))
        (Chord("<b' d''>8"), Chord("<c'' e''>8"))

    Return generator.
    '''
    from abjad.tools import componenttools

    current_group = ()
    for group in componenttools.yield_topmost_components_grouped_by_type(sequence):
        if type(group[0]) in klasses:
            current_group = current_group + group
        elif current_group:
            yield current_group
            current_group = ()

    if current_group:
        yield current_group



def is_integer_n_tuple(expr, n):
    r'''.. versionadded:: 2.9

    True when `expr` is an integer tuple of length `n`::

        >>> sequencetools.is_integer_n_tuple((19, 20, 21), 3)
        True

    Otherwise false::

        >>> sequencetools.is_integer_n_tuple((19, 20, 'text'), 3)
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and len(expr) == n and all([isinstance(x, int) for x in expr])



from abjad import *
from abjad.tools import introspectiontools


def test_introspectiontools_get_current_function_name_01():

    def bar():
        return introspectiontools.get_current_function_name()

    assert bar() == 'bar'


def test_introspectiontools_get_current_function_name_02():

    def foo():
        return introspectiontools.get_current_function_name()

    assert foo() == 'foo'



from abjad.tools import mathtools
from abjad.tools import sequencetools
from abjad.tools.rhythmmakertools.BurnishedRhythmMaker import BurnishedRhythmMaker


class DivisionBurnishedTaleaRhythmMaker(BurnishedRhythmMaker):
    r'''.. versionadded:: 2.8

    Division-burnished talea rhythm-maker.

    Configure the rhythm-maker at instantiation::

        >>> talea, talea_denominator, prolation_addenda = [1, 1, 2, 4], 32, [0, 3]
        >>> lefts, middles, rights = [-1], [0], [-1]
        >>> left_lengths, right_lengths = [1], [1]
        >>> secondary_divisions = [14]
        >>> maker = rhythmmakertools.DivisionBurnishedTaleaRhythmMaker(
        ... talea, talea_denominator, prolation_addenda, lefts, middles, rights, 
        ... left_lengths, right_lengths, secondary_divisions)

    Then call the rhythm-maker on any sequence of divisions::

        >>> divisions = [(5, 16), (6, 16)]
        >>> music = maker(divisions)

    The resulting Abjad objects can then be included in any score and the rhythm
    maker can be called again and again on different divisions::

        >>> music = sequencetools.flatten_sequence(music)
        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, music)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 5/16
                {
                    r32
                    c'32
                    c'16
                    c'8
                    c'32
                    r32
                }
            }
            {
                \time 6/16
                \times 4/7 {
                    r16
                    c'8
                    r32
                }
                {
                    r32
                    c'16
                    c'8
                    r32
                }
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### PRIVATE METHODS ###

    def _burnish_division_parts(self, divisions, quintuplet):
        lefts, middles, rights, left_lengths, right_lengths=quintuplet
        lefts_index, rights_index = 0, 0
        burnished_divisions = []
        for division_index, division in enumerate(divisions):
            left_length = left_lengths[division_index]
            left = lefts[lefts_index:lefts_index+left_length]
            lefts_index += left_length
            right_length = right_lengths[division_index]
            right = rights[rights_index:rights_index+right_length]
            rights_index += right_length
            available_left_length = len(division)
            left_length = min([left_length, available_left_length])
            available_right_length = len(division) - left_length
            right_length = min([right_length, available_right_length])
            middle_length = len(division) - left_length - right_length

            left = left[:left_length]
            middle = middle_length * [middles[division_index]]
            right = right[:right_length]
            left_part, middle_part, right_part = \
                sequencetools.partition_sequence_by_counts(
                division, [left_length, middle_length, right_length], cyclic=False, overhang=False)
            left_part = self._burnish_division_part(left_part, left)
            middle_part = self._burnish_division_part(middle_part, middle)
            right_part = self._burnish_division_part(right_part, right)
            burnished_division = left_part + middle_part + right_part
            burnished_divisions.append(burnished_division)
        unburnished_weights = [mathtools.weight(x) for x in divisions]
        burnished_weights = [mathtools.weight(x) for x in burnished_divisions]
        assert burnished_weights == unburnished_weights
        return burnished_divisions



from abjad.tools.contexttools.Context import Context


class Staff(Context):
    r'''Abjad model of a staff:

    ::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> f(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
        }


    ::

        >>> show(staff) # doctest: +SKIP

    Return Staff instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='Staff', name=None):
        Context.__init__(self, music=music)
        self._initialize_keyword_values(context_name=context_name, name=name)



def get_markup_attached_to_component(component):
    r'''.. versionadded:: 2.0

    Get markup attached to `component`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> slur = spannertools.SlurSpanner(staff[:])

    ::

        >>> markuptools.Markup('foo')(staff[0])
        Markup(('foo',))(c'8)

    ::

        >>> markuptools.Markup('bar')(staff[0])
        Markup(('bar',))(c'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 (
                - \markup {
                    \column
                        {
                            foo
                            bar
                        }
                    }
            d'8
            e'8
            f'8 )
        }

    ::

        >>> markuptools.get_markup_attached_to_component(staff[0])
        (Markup(('foo',))(c'8), Markup(('bar',))(c'8))

    Return tuple of zero or more markup objects.
    '''
    from abjad.tools import markuptools

    result = []
    for mark in component._marks_for_which_component_functions_as_start_component:
        if isinstance(mark, markuptools.Markup):
            result.append(mark)

    result = tuple(result)
    return result



from abjad.tools import pitchtools
from abjad.tools.spannertools.Spanner import Spanner


class TrillSpanner(Spanner):
    r'''Abjad trill spanner::

        >>> staff = Staff("c'8 d'8 e'8 f'8")

    ::

        >>> spannertools.TrillSpanner(staff[:])
        TrillSpanner(c'8, d'8, e'8, f'8)

    ::

        >>> f(staff)
        \new Staff {
            c'8 \startTrillSpan
            d'8
            e'8
            f'8 \stopTrillSpan
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Override LilyPond TrillSpanner grob.

    Return trill spanner.
    '''

    ### INITIALIZER ###

    def __init__(self, components=None):
        Spanner.__init__(self, components)
        self._pitch = None

    ### PRIVATE METHODS ###

    def _copy_keyword_args(self, new):
        new.written_pitch = self.written_pitch

    def _format_before_leaf(self, leaf):
        result = []
        if self.pitch is not None:
            if self._is_my_first_leaf(leaf):
                result.append(r'\pitchedTrill')
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.append(r'\startTrillSpan')
            if self.pitch is not None:
                result.append(str(self.pitch))
        if self._is_my_last_leaf(leaf):
            result.append(r'\stopTrillSpan')
        return result

    ### PUBLIC PROPERTIES ###

    @apply
    def pitch():
        def fget(self):
            r'''Optional read / write pitch for pitched trills.

                ::

                    >>> t = Staff("c'8 d'8 e'8 f'8")
                    >>> trill = spannertools.TrillSpanner(t[:2])
                    >>> trill.pitch = pitchtools.NamedChromaticPitch('cs', 4)

                ::

                    >>> f(t)
                    \new Staff {
                        \pitchedTrill c'8 \startTrillSpan cs'
                        d'8 \stopTrillSpan
                        e'8
                        f'8
                    }

            Set pitch.
            '''
            return self._pitch
        def fset(self, expr):
            if expr is None:
                self._pitch = expr
            else:
                pitch = pitchtools.NamedChromaticPitch(expr)
                self._pitch = pitch
        return property(**locals())

    @apply
    def written_pitch():
        def fget(self):
            return self.pitch
        def fset(self, arg):
            self.pitch = arg
        return property(**locals())



from abjad.tools import voicetools


def iterate_voices_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.0

    Iterate voices forward in `expr`::

        >>> voice_1 = Voice("c'8 d'8 e'8 f'8")
        >>> voice_2 = Voice("c'4 b4")
        >>> staff = Staff([voice_1, voice_2])
        >>> staff.is_parallel = True

    ::

        >>> f(staff)
        \new Staff <<
            \new Voice {
                c'8
                d'8
                e'8
                f'8
            }
            \new Voice {
                c'4
                b4
            }
        >>

    ::

        >>> for voice in iterationtools.iterate_voices_in_expr(staff):
        ...   voice
        Voice{4}
        Voice{2}

    Iterate voices backward in `expr`::

    ::

        >>> for voice in iterationtools.iterate_voices_in_expr(staff, reverse=True):
        ...   voice
        Voice{2}
        Voice{4}

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=voicetools.Voice, reverse=reverse, start=start, stop=stop)



import types
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTLineageDirective(ReSTDirective):
    '''An ReST Abjad lineage diagram directive:

    ::

        >>> documentationtools.ReSTLineageDirective(argument=beamtools.BeamSpanner)
        ReSTLineageDirective(
            argument='abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner'
            )

    ::

        >>> print _.rest_format
        .. abjad-lineage:: abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner

    Return `ReSTLineageDirective` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, name=None, options=None):
        if isinstance(argument, types.TypeType):
            argument = argument.__module__ + '.' + argument.__name__
        ReSTDirective.__init__(self, argument=argument, children=children, name=name, options=options)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def directive(self):
        return 'abjad-lineage'



from abjad.tools import skiptools


def fill_measures_in_expr_with_full_measure_spacer_skips(expr, iterctrl=None):
    '''.. versionadded:: 1.1

    Fill measures in `expr` with full-measure spacer skips.
    '''
    from abjad.tools import contexttools
    from abjad.tools import iterationtools
    from abjad.tools.spannertools._withdraw_component_from_attached_spanners import \
        _withdraw_component_from_attached_spanners

    if iterctrl is None:
        iterctrl = lambda measure, i: True

    for i, measure in enumerate(iterationtools.iterate_measures_in_expr(expr)):
        if iterctrl(measure, i):
            skip = skiptools.Skip(1)
            # allow zero-update iteration
            time_siganture = contexttools.get_effective_time_signature(measure)
            #skip.duration_multiplier = time_siganture.duration / time_siganture.multiplier
            skip.duration_multiplier = time_siganture.duration / time_siganture.implied_prolation
            measure[:] = [skip]
            _withdraw_component_from_attached_spanners(measure)



# -*- encoding: utf-8 -*-

def list_known_languages():
    '''List all note-input languages recognized by ``LilyPondParser``:

    ::

        >>> for x in lilypondparsertools.list_known_languages():
        ...     print x
        ...
        catalan
        deutsch
        english
        espanol
        español
        français
        italiano
        nederlands
        norsk
        portugues
        suomi
        svenska
        vlaams

    Return list.
    '''
    from abjad.ly import language_pitch_names

    return sorted(language_pitch_names.keys())



from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import markuptools
from abjad.tools import notetools


def label_leaves_in_expr_with_pitch_class_numbers(expr, number=True, color=False,
    markup_direction=Down):
    r'''.. versionadded:: 1.1

    Label leaves in `expr` with pitch-class numbers::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> labeltools.label_leaves_in_expr_with_pitch_class_numbers(staff)
        >>> print staff.lilypond_format
        \new Staff {
            c'8 _ \markup { \small 0 }
            d'8 _ \markup { \small 2 }
            e'8 _ \markup { \small 4 }
            f'8 _ \markup { \small 5 }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    When ``color=True`` call 
    :func:`~abjad.tools.labeltools.color_note_head_by_numbered_chromatic_pitch_class_color_map`::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> labeltools.label_leaves_in_expr_with_pitch_class_numbers(
        ...     staff, color=True, number=False)
        >>> print staff.lilypond_format
        \new Staff {
            \once \override NoteHead #'color = #(x11-color 'red)
            c'8
            \once \override NoteHead #'color = #(x11-color 'orange)
            d'8
            \once \override NoteHead #'color = #(x11-color 'ForestGreen)
            e'8
            \once \override NoteHead #'color = #(x11-color 'MediumOrchid)
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    You can set `number` and `color` at the same time.

    .. versionchanged:: 2.0
        renamed ``label.leaf_pcs()`` to
        ``labeltools.label_leaves_in_expr_with_pitch_class_numbers()``.

    Return none.
    '''
    from abjad.tools import labeltools

    for note in iterationtools.iterate_notes_in_expr(expr):
        if number:
            label = markuptools.MarkupCommand(
                'small', str(abs(note.written_pitch.numbered_chromatic_pitch_class)))
            markuptools.Markup(label, markup_direction)(note)
        if color:
            labeltools.color_note_head_by_numbered_chromatic_pitch_class_color_map(note)



from abjad.tools import componenttools


def all_are_voices(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad voices::

        >>> voice = Voice("c'4 d'4 e'4 f'4")

    ::

        >>> voicetools.all_are_voices([voice])
        True

    True when `expr` is an empty sequence::

        >>> voicetools.all_are_voices([])
        True

    Otherwise false::

        >>> voicetools.all_are_voices('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''

    from abjad.tools import voicetools

    return componenttools.all_are_components(expr, klasses=(voicetools.Voice,))



from abjad.tools import configurationtools
from abjad.tools import iotools
from abjad.tools.developerscripttools.DirectoryScript import DirectoryScript
from abjad.tools.developerscripttools.SvnMessageScript import SvnMessageScript
import os


class SvnCommitScript(DirectoryScript):
    '''Run `svn commit`, using the commit message stored in the `.abjad` directory.

    The commit message will be printed to the terminal, and must be manually
    accepted or rejected before proceeding:

    ::

        bash$ ajv svn ci -h
        usage: svn-commit [-h] [--version] path

        "svn commit", using previously written commit message.

        positional arguments:
          path        commit the path PATH

        optional arguments:
          -h, --help  show this help message and exit
          --version   show program's version number and exit

    Return `SvnCommitScript` instance.
    '''

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 'ci'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return 'svn'

    @property
    def short_description(self):
        return '"svn commit", using previously written commit message.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC METHODS ###

    def process_args(self, args):
        from abjad import ABJCFG
        commit_file = 'abjad_commit.txt'
        commit_path = os.path.join(ABJCFG.ABJAD_CONFIG_DIRECTORY_PATH, commit_file)

        while not os.path.exists(commit_path):
            SvnMessageScript()()

        commit_command = 'svn commit {} -F {}'.format(args.path, commit_path)

        while True:
            print '\nSTORED COMMIT MESSAGE:\n'
            with open(commit_path, 'r') as f:
                print f.read()
            result = raw_input('Accept [Y], Reject [n], Abort [a]: ').strip().lower()
            if result in ('', 'y', 'yes'):
                iotools.spawn_subprocess(commit_command)
                return
            elif result in ('n', 'no'):
                SvnMessageScript()()
            elif result in ('a', 'abort'):
                return
            else:
                print 'Invalid response {!r}, repeating...'.format(result)
            

    def setup_argument_parser(self, parser):
        parser.add_argument('path',
            help='commit the path PATH',
            type=self._validate_path,
            )




from abjad import *
from abjad.demos.mozart.choose_mozart_measures import choose_mozart_measures
from abjad.demos.mozart.make_mozart_measure import make_mozart_measure


def make_mozart_score():

    score_template = scoretemplatetools.TwoStaffPianoScoreTemplate()
    score = score_template()

    # select the measures to use
    choices = choose_mozart_measures()

    # create and populate the volta containers
    treble_volta = Container()
    bass_volta = Container()
    for choice in choices[:7]:
        treble, bass = make_mozart_measure(choice)
        treble_volta.append(treble)
        bass_volta.append(bass)

    # add marks to the volta containers
    marktools.LilyPondCommandMark(
        'repeat volta 2', 'before'
        )(treble_volta)
    marktools.LilyPondCommandMark(
        'repeat volta 2', 'before'
        )(bass_volta)

    # add the volta containers to our staves
    score['RH Voice'].append(treble_volta)
    score['LH Voice'].append(bass_volta)

    # create and populate the alternative ending containers
    treble_alternative = Container()
    bass_alternative = Container()
    for choice in choices[7:9]:
        treble, bass = make_mozart_measure(choice)
        treble_alternative.append(treble)
        bass_alternative.append(bass)

    # add marks to the alternative containers
    marktools.LilyPondCommandMark(
        'alternative', 'before'
        )(treble_alternative)
    marktools.LilyPondCommandMark(
        'alternative', 'before'
        )(bass_alternative)

    # add the alternative containers to our staves
    score['RH Voice'].append(treble_alternative)
    score['LH Voice'].append(bass_alternative)

    # create the remaining measures
    for choice in choices[9:]:
        treble, bass = make_mozart_measure(choice)
        score['RH Voice'].append(treble)
        score['LH Voice'].append(bass)

    # add marks
    contexttools.TimeSignatureMark((3, 8))(score['RH Staff'])
    marktools.BarLine('|.')(score['RH Voice'][-1])
    marktools.BarLine('|.')(score['LH Voice'][-1])

    # remove the old, default Piano InstrumentMark attached to the PianoStaff
    # and add a custom instrument mark
    contexttools.detach_instrument_marks_attached_to_component(score['Piano Staff'])
    contexttools.InstrumentMark(
        'Katzenklavier', 'kk.',
        target_context = scoretools.PianoStaff
        )(score['Piano Staff'])

    return score



from abjad.tools import markuptools
from abjad.tools import notetools
from abjad.tools import pitchtools
from abjad.tools import verticalitytools


def label_vertical_moments_in_expr_with_chromatic_intervals(expr, markup_direction=Down):
    r'''.. versionadded:: 2.0

    Label harmonic chromatic intervals of every vertical moment in `expr`::

        >>> score = Score([])
        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "alto" g4 f4""")
        >>> score.append(staff)
        >>> staff = Staff(r"""\clef "bass" c,2""")
        >>> score.append(staff)

    ::

        >>> labeltools.label_vertical_moments_in_expr_with_chromatic_intervals(
        ...     score)

    ::

        >>> f(score)
        \new Score <<
            \new Staff {
                c'8
                d'8
                    _ \markup {
                        \small
                            \column
                                {
                                    26
                                    19
                                }
                        }
                e'8
                f'8
                    _ \markup {
                        \small
                            \column
                                {
                                    29
                                    17
                                }
                        }
            }
            \new Staff {
                \clef "alto"
                g4
                f4
                    _ \markup {
                        \small
                            \column
                                {
                                    28
                                    17
                                }
                        }
            }
            \new Staff {
                \clef "bass"
                c,2
                    _ \markup {
                        \small
                            \column
                                {
                                    24
                                    19
                                }
                        }
            }
        >>

    ::

        >>> show(score) # doctest: +SKIP

    Return none.
    '''

    for vertical_moment in verticalitytools.iterate_vertical_moments_in_expr(expr):
        leaves = vertical_moment.leaves
        notes = [leaf for leaf in leaves if isinstance(leaf, notetools.Note)]
        if not notes:
            continue
        notes.sort(lambda x, y: cmp(x.written_pitch.numbered_chromatic_pitch,
            y.written_pitch.numbered_chromatic_pitch))
        notes.reverse()
        bass_note = notes[-1]
        upper_notes = notes[:-1]
        hcis = []
        for upper_note in upper_notes:
            hci = pitchtools.calculate_harmonic_chromatic_interval(
                bass_note, upper_note)
            hcis.append(hci)
        hcis = ' '.join([str(hci) for hci in hcis])
        hcis = r'\small \column { %s }' % hcis
        markuptools.Markup(hcis, markup_direction)(vertical_moment.start_leaves[-1])



from abjad.tools import durationtools


def clip_interval_durations_to_range(intervals, minimum=None, maximum=None):
    from abjad.tools import timeintervaltools

    assert timeintervaltools.all_are_intervals_or_trees_or_empty(intervals)

    if minimum is not None:
        minimum = durationtools.Duration(minimum)
        assert 0 < minimum

    if maximum is not None:
        maximum = durationtools.Duration(maximum)
        assert 0 < maximum

    if minimum is not None and maximum is not None:
        assert minimum < maximum

    if isinstance(intervals, timeintervaltools.TimeIntervalTree):
        tree = intervals
    else:
        tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return tree

    intervals = []
    for interval in tree:
        if minimum is not None and interval.duration < minimum:
            intervals.append(interval.scale_to_rational(minimum))
        elif maximum is not None and maximum < interval.duration:
            intervals.append(interval.scale_to_rational(maximum))
        else:
            intervals.append(interval)

    return timeintervaltools.TimeIntervalTree(intervals)



from abjad.tools import containertools


def iterate_containers_in_expr(expr, reverse=False, start=0, stop=None):
    r'''.. versionadded:: 2.10

    Iterate containers forward in `expr`:

    ::

        >>> staff = Staff([Voice("c'8 d'8"), Voice("e'8 f'8 g'8")])
        >>> Tuplet(Fraction(2, 3), staff[1][:])
        Tuplet(2/3, [e'8, f'8, g'8])
        >>> staff.is_parallel = True

    ::

        >>> f(staff)
        \new Staff <<
            \new Voice {
                c'8
                d'8
            }
            \new Voice {
                \times 2/3 {
                    e'8
                    f'8
                    g'8
                }
            }
        >>

    ::

        >>> for x in iterationtools.iterate_containers_in_expr(staff):
        ...   x
        Staff<<2>>
        Voice{2}
        Voice{1}
        Tuplet(2/3, [e'8, f'8, g'8])

    Iterate containers backward in `expr`:

    ::

        >>> for x in iterationtools.iterate_containers_in_expr(staff, reverse=True):
        ...   x
        Staff<<2>>
        Voice{1}
        Tuplet(2/3, [e'8, f'8, g'8])
        Voice{2}

    Ignore threads.

    Return generator.
    '''
    from abjad.tools import iterationtools

    return iterationtools.iterate_components_in_expr(
        expr, klass=containertools.Container, reverse=reverse, start=start, stop=stop)



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import mathtools


def duration_and_possible_denominators_to_time_signature(duration, denominators=None, factor=None):
    '''Make new time signature equal to `duration`::

        >>> timesignaturetools.duration_and_possible_denominators_to_time_signature(
        ...     Duration(3, 2))
        TimeSignatureMark((3, 2))

    Make new time signature equal to `duration` with denominator equal to the
    first possible element in `denominators`::

        >>> timesignaturetools.duration_and_possible_denominators_to_time_signature(
        ...     Duration(3, 2), denominators=[5, 6, 7, 8])
        TimeSignatureMark((9, 6))

    Make new time signature equal to `duration` with denominator divisible by
    `factor`::

        >>> timesignaturetools.duration_and_possible_denominators_to_time_signature(
        ...     Duration(3, 2), factor=5)
        TimeSignatureMark((15, 10))

    .. note:: possibly divide this into two separate functions?

    Return new time signature.
    '''

    # check input
    duration = durationtools.Duration(duration)

    if denominators is not None:
        if factor is not None:
            denominators = [d for d in denominators if factor in mathtools.factors(d)]
        for desired_denominator in sorted(denominators):
            nonreduced_fraction = mathtools.NonreducedFraction(duration)
            candidate_pair = nonreduced_fraction.with_denominator(desired_denominator)
            if candidate_pair.denominator == desired_denominator:
                return contexttools.TimeSignatureMark(candidate_pair)
    if factor is not None:
        if factor in mathtools.factors(duration.denominator):
            return contexttools.TimeSignatureMark(duration)
        else:
            time_signature_numerator = factor * duration.numerator
            time_signature_denominator = factor * duration.denominator
            return contexttools.TimeSignatureMark((time_signature_numerator, time_signature_denominator))
    else:
        return contexttools.TimeSignatureMark(duration)



from abjad.tools import componenttools
from abjad.tools import decoratortools


@decoratortools.requires(object)
def all_are_chords(expr):
    '''.. versionadded:: 2.6

    True when `expr` is a sequence of Abjad chords::

        >>> chords = [Chord("<c' e' g'>4"), Chord("<c' f' a'>4")]

    ::

        >>> chordtools.all_are_chords(chords)
        True

    True when `expr` is an empty sequence::

        >>> chordtools.all_are_chords([])
        True

    Otherwise false::

        >>> chordtools.all_are_chords('foo')
        False

    Return boolean.

    Function wraps ``componenttools.all_are_components()``.
    '''
    from abjad.tools import chordtools

    return componenttools.all_are_components(expr, klasses=(chordtools.Chord,))



def is_null_tuple(expr):
    r'''.. versionadded:: 2.9

    True when `expr` is a tuple of length ``0``::

        >>> sequencetools.is_null_tuple(())
        True

    Otherwise false::

        >>> sequencetools.is_null_tuple((19, 20, 21))
        False

    Return boolean.
    '''

    return isinstance(expr, tuple) and not len(expr)



from abjad.tools import componenttools


# TODO: change interface to fuse.containers_by_name() to pass containers-to-be-fused explicitly.
def fuse_like_named_contiguous_containers_in_expr(expr):
    r'''Fuse like-named contiguous containers in `expr`::

        >>> staff = Staff(r"\new Voice { c'8 d'8 } \new Voice { e'8 f'8 }")
        >>> staff[0].name = 'soprano'
        >>> staff[1].name = 'soprano'

    ::

        >>> f(staff)
        \new Staff {
            \context Voice = "soprano" {
                c'8
                d'8
            }
            \context Voice = "soprano" {
                e'8
                f'8
            }
        }

    ::

        >>> containertools.fuse_like_named_contiguous_containers_in_expr(staff)
        Staff{1}

    ::

        >>> f(staff)
        \new Staff {
            \context Voice = "soprano" {
                c'8
                d'8
                e'8
                f'8
            }
        }

    Return `expr`.

    .. versionchanged:: 2.0
        renamed ``fuse.containers_by_reference()`` to
        ``containertools.fuse_like_named_contiguous_containers_in_expr()``.
    '''
    from abjad.tools import containertools
    from abjad.tools import iterationtools
    from abjad.tools import tuplettools

    merged = False
    if not isinstance(expr, list):
        expr = [expr]
    expr = containertools.Container(expr)

    g = iterationtools.iterate_components_depth_first(expr, direction=Right)
    for component in g:
        next_component = componenttools.get_nth_namesake_from_component(component, 1)
        if isinstance(next_component, containertools.Container) and \
            not next_component.is_parallel and \
            not isinstance(next_component, tuplettools.Tuplet) and \
            component.parentage.root is next_component.parentage.root:
                component.extend(next_component)
                componenttools.remove_component_subtree_from_score_and_spanners([next_component])
                merged = True
    if merged:
        containertools.remove_leafless_containers_in_expr(expr)
        return expr.pop(0)



def all_intervals_are_nonoverlapping(intervals):
    '''True when all intervals in `intervals` in tree are non-overlapping.
    '''

    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    for i in range(1, len(tree)):
        if tree[i].start < tree[i -1].stop:
            return False

    return True



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import mathtools


# TODO: implement measuretools.set_measure_denominator_and_adjust_contents().
def set_measure_denominator_and_adjust_numerator(measure, denominator):
    r'''.. versionadded:: 1.1

    Set `measure` time signature `denominator` and multiply time signature numerator accordingly::

        >>> measure = Measure((3, 8), "c'8 d'8 e'8")
        >>> beamtools.BeamSpanner(measure.leaves)
        BeamSpanner(c'8, d'8, e'8)

    ::

        >>> f(measure)
        {
            \time 3/8
            c'8 [
            d'8
            e'8 ]
        }

    ::

        >>> measuretools.set_measure_denominator_and_adjust_numerator(measure, 16)
        Measure(6/16, [c'8, d'8, e'8])

    ::

        >>> f(measure)
        {
            \time 6/16
            c'8 [
            d'8
            e'8 ]
        }

    Leave `measure` contents unchanged.

    Return `measure`.

    .. versionchanged:: 2.0
        renamed ``measuretools.set_measure_denominator_and_multiply_numerator()`` to
        ``measuretools.set_measure_denominator_and_adjust_numerator()``.
    '''
    from abjad.tools import measuretools

    if isinstance(measure, measuretools.Measure):
        # to allow iteration inside zero-update loop
        old_time_signature = contexttools.get_effective_time_signature(measure)
        old_time_signature_pair = (old_time_signature.numerator, old_time_signature.denominator)
        new_time_signature = mathtools.NonreducedFraction(old_time_signature_pair)
        new_time_signature = new_time_signature.with_denominator(denominator)
        new_time_signature = contexttools.TimeSignatureMark(new_time_signature)
        contexttools.detach_time_signature_marks_attached_to_component(measure)
        new_time_signature.attach(measure)

    return measure



'''Time signature manipulation tools.

   Modules in this package depend on, may freely import from, the following:

      from abjad.tools import durationtools
      from abjad.tools import mathtools
'''

from abjad.tools import importtools

importtools.import_structured_package(__path__[0], globals(), package_root_name='abjad')

_documentation_section = 'core'



import fractions
from abjad.tools import mathtools


def group_nonreduced_fractions_by_implied_prolation(durations):
    '''.. versionadded:: 1.1

    Group `durations` by implied prolation::

        >>> durations = [(1, 4), (1, 8), (1, 3), (1, 6), (1, 4)]

    ::

        >>> for group in durationtools.group_nonreduced_fractions_by_implied_prolation(durations):
        ...     group
        [NonreducedFraction(1, 4), NonreducedFraction(1, 8)]
        [NonreducedFraction(1, 3), NonreducedFraction(1, 6)]
        [NonreducedFraction(1, 4)]

    Return list of duration lists.
    '''
    from abjad.tools import durationtools

    durations = [mathtools.NonreducedFraction(duration) for duration in durations]
    assert 0 < len(durations)

    group = [durations[0]]
    result = [group]
    for d in durations[1:]:
        d_f = set(mathtools.factors(d.denominator))
        d_f.discard(2)
        gd_f = set(mathtools.factors(group[0].denominator))
        gd_f.discard(2)
        if d_f == gd_f:
            group.append(d)
        else:
            group = [d]
            result.append(group)
    return result



import copy

from abjad.tools import chordtools
from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import marktools
from abjad.tools import measuretools
from abjad.tools import notetools
from abjad.tools import resttools
from abjad.tools import tietools

from abjad.demos.part.create_pitch_contour_reservoir import create_pitch_contour_reservoir
from abjad.demos.part.durate_pitch_contour_reservoir import durate_pitch_contour_reservoir
from abjad.demos.part.shadow_pitch_contour_reservoir import shadow_pitch_contour_reservoir
from abjad.demos.part.edit_first_violin_voice import edit_first_violin_voice
from abjad.demos.part.edit_second_violin_voice import edit_second_violin_voice
from abjad.demos.part.edit_viola_voice import edit_viola_voice
from abjad.demos.part.edit_cello_voice import edit_cello_voice
from abjad.demos.part.edit_bass_voice import edit_bass_voice



def add_string_music_to_score(score):

    # generate some pitch and rhythm information
    pitch_contour_reservoir = create_pitch_contour_reservoir()
    shadowed_contour_reservoir = shadow_pitch_contour_reservoir(
        pitch_contour_reservoir)
    durated_reservoir = durate_pitch_contour_reservoir(
        shadowed_contour_reservoir)

    # add six dotted-whole notes and the durated contours to each string voice
    for instrument_name, descents in durated_reservoir.iteritems():
        instrument_voice = score['%s Voice' % instrument_name]
        instrument_voice.extend("R1. R1. R1. R1. R1. R1.")
        for descent in descents:
            instrument_voice.extend(descent)

    # apply instrument-specific edits
    edit_first_violin_voice(score, durated_reservoir)
    edit_second_violin_voice(score, durated_reservoir)
    edit_viola_voice(score, durated_reservoir)
    edit_cello_voice(score, durated_reservoir)
    edit_bass_voice(score, durated_reservoir)

    # chop all string parts into 6/4 measures
    for voice in iterationtools.iterate_voices_in_expr(score['Strings Staff Group']):
        for shard in componenttools.split_components_at_offsets(voice[:],
            [(6, 4)], cyclic=True):
            measuretools.Measure((6, 4), shard)



def set_ascending_named_chromatic_pitches_on_tie_chains_in_expr(expr):
    r'''.. versionadded:: 1.1

    Set ascending named chromatic pitches on nontied pitched components in `expr`::

        >>> voice = Voice(notetools.make_notes(0, [(5, 32)] * 4))
        >>> pitchtools.set_ascending_named_chromatic_pitches_on_tie_chains_in_expr(voice)

    ::

        >>> f(voice)
        \new Voice {
            c'8 ~
            c'32
            cs'8 ~
            cs'32
            d'8 ~
            d'32
            ef'8 ~
            ef'32
        }

    Used primarily in generating test file examples.

    Return none.

    .. versionchanged:: 2.0
        renamed ``pitchtools.chromaticize()`` to
        ``pitchtools.set_ascending_named_chromatic_pitches_on_tie_chains_in_expr()``.
    '''
    from abjad.tools import chordtools
    from abjad.tools import notetools
    from abjad.tools import tietools

    for i, tie_chain in enumerate(tietools.iterate_tie_chains_in_expr(expr)):
        pitch = i
        if isinstance(tie_chain[0], notetools.Note):
            for note in tie_chain:
                note.written_pitch = pitch
        elif isinstance(tie_chain[0], chordtools.Chord):
            for chord in tie_chain:
                chord.written_pitches = [pitch]



def get_proper_descendents_of_component(component):
    r'''.. versionadded:: 2.9

    Get proper descendents of `component`::

        >>> staff = Staff(r"c'4 \times 2/3 { d'8 e'8 f'8 }")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            \times 2/3 {
                d'8
                e'8
                f'8
            }
        }

    ::

        >>> componenttools.get_proper_descendents_of_component(staff)
        [Note("c'4"), Tuplet(2/3, [d'8, e'8, f'8]), Note("d'8"), Note("e'8"), Note("f'8")]

    Return list of proper descendents of `component`.
    '''
    from abjad.tools import iterationtools

    # create list of component and all descendents of component
    result = list(iterationtools.iterate_components_in_expr(component))

    # remove component from list
    result.remove(component)

    # return result
    return result



from abjad.tools import mathtools


def get_nth_component_in_time_order_from_component(component, n):
    r'''.. versionadded:: 2.9

    Get nth component from `component` in temporal order::

        >>> staff = Staff(r"c'4 \times 2/3 { d'8 e'8 f'8 } g'2")

    ::

        >>> f(staff)
        \new Staff {
            c'4
            \times 2/3 {
                d'8
                e'8
                f'8
            }
            g'2
        }

    ::

        >>> staff.leaves[1]
        Note("d'8")

    Return component right of `component` for positive `n`::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff.leaves[1], 1)
        Note("e'8")

    ::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff.leaves[1], 2)
        Note("f'8")

    ::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff.leaves[1], 3)
        Note("g'2")

    Return component left of `component` for negative `n`::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff.leaves[1], -1)
        Note("c'4")

    Return `component` when `n` is ``0``::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff.leaves[1], 0)
        Note("d'8")

    Return none when `n` is out of range::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff.leaves[1], 99) is None
        True

    Return none when `component` has no parent::

        >>> componenttools.get_nth_component_in_time_order_from_component(
        ...     staff, 1) is None
        True

    Return component or none.
    '''
    from abjad.tools import componenttools
    
    assert isinstance(component, componenttools.Component)
    assert mathtools.is_integer_equivalent_expr(n)

    def _next(component):
        for parent in componenttools.get_improper_parentage_of_component(component):
            next_sibling = componenttools.get_nth_sibling_from_component(parent, 1)
            if next_sibling is not None:
                return next_sibling

    def _prev(component):
        for parent in componenttools.get_improper_parentage_of_component(component):
            next_sibling = componenttools.get_nth_sibling_from_component(parent, -1)
            if next_sibling is not None:
                return next_sibling

    result = component

    if 0 < n:
        for i in range(n):
            result = _next(result)
    elif n < 0:
        for i in range(abs(n)):
            result = _prev(result)

    return result 



import pickle
from abjad import *

def test_GraphvizGraph_pickle_01():

    graph = documentationtools.GraphvizGraph()
    graph.append(documentationtools.GraphvizSubgraph())
    graph[0].append(documentationtools.GraphvizNode())
    graph[0].append(documentationtools.GraphvizNode())
    graph[0].append(documentationtools.GraphvizNode())
    graph[0].append(documentationtools.GraphvizSubgraph())
    graph[0][-1].append(documentationtools.GraphvizNode())
    graph.append(documentationtools.GraphvizNode())
    edge = documentationtools.GraphvizEdge()(graph[0][1], graph[1])
    edge = documentationtools.GraphvizEdge()(graph[0][0], graph[0][-1][0])

    r'''
    digraph Graph {
        subgraph cluster_0 {
            node_0_0;
            node_0_1;
            node_0_2;
            subgraph cluster_0_3 {
                node_0_3_0;
            }
            node_0_0 -> node_0_3_0;
        }
        node_1;
        node_0_1 -> node_1;
    }
    '''

    pickled = pickle.loads(pickle.dumps(graph))

    assert graph.graphviz_format == pickled.graphviz_format



def is_immediate_temporal_successor_of_component(component, expr):
    r'''.. versionadded:: 2.9

    True when `expr` is immediate temporal successor of `component`.

    Otherwise false.
    '''
    from abjad.tools import componenttools

    temporal_successors = []
    cur = component
    while cur is not None:
        next_sibling = componenttools.get_nth_sibling_from_component(cur, 1)
        if next_sibling is None:
            cur = cur.parent
        else:
            temporal_successors = \
                componenttools.get_improper_descendents_of_component_that_start_with_component(next_sibling)
            break

    return expr in temporal_successors



from abjad import *


def test_instrumenttools_MezzoSopranoVoice_storage_format_01():

    voice = instrumenttools.MezzoSopranoVoice()

    assert voice.storage_format == 'instrumenttools.MezzoSopranoVoice()'



from abjad.tools import componenttools
from abjad.tools import iterationtools
from abjad.tools import markuptools


def label_leaves_in_expr_with_tuplet_depth(expr, markup_direction=Down):
    r'''.. versionadded:: 1.1

    Label leaves in `expr` with tuplet depth::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8")
        >>> tuplettools.FixedDurationTuplet(Duration(2, 8), staff[-3:])
        FixedDurationTuplet(1/4, [e'8, f'8, g'8])
        >>> labeltools.label_leaves_in_expr_with_tuplet_depth(staff)
        >>> f(staff)
        \new Staff {
            c'8 _ \markup { \small 0 }
            d'8 _ \markup { \small 0 }
            \times 2/3 {
                e'8 _ \markup { \small 1 }
                f'8 _ \markup { \small 1 }
                g'8 _ \markup { \small 1 }
            }
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return none.

    .. versionchanged:: 2.0
        renamed ``label.leaf_depth_tuplet()`` to
        ``labeltools.label_leaves_in_expr_with_tuplet_depth()``.
    '''

    for leaf in iterationtools.iterate_leaves_in_expr(expr):
        label = markuptools.MarkupCommand('small', str(leaf.parentage.tuplet_depth))
        markuptools.Markup(label, markup_direction)(leaf)



from abjad import *


def test_tuplettools_leaf_to_tuplet_with_ratio_01():

    note = Note(0, (3, 16))

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1], is_diminution=False)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2], is_diminution=False)
    assert t.lilypond_format == "{\n\tc'16\n\tc'8\n}"

    # TODO: DECIDE ON DOTTED VALUES #
    #t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2], is_diminution=False)
    #assert t.lilypond_format == "\\fraction \\times 6/5 {\n\tc'32\n\tc'16\n\tc'16\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2, 3], is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 3/2 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2, 3, 3], is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 12/11 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n\tc'32.\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2, 3, 3, 4], is_diminution=False)
    assert t.lilypond_format == "\\fraction \\times 8/5 {\n\tc'128\n\tc'64\n\tc'64\n\tc'64.\n\tc'64.\n\tc'32\n}"


def test_tuplettools_leaf_to_tuplet_with_ratio_02():

    note = Note(0, (3, 16))

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1], is_diminution=True)
    assert t.lilypond_format == "{\n\tc'8.\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2], is_diminution=True)
    assert t.lilypond_format == "{\n\tc'16\n\tc'8\n}"

    # TODO: DECIDE ON DOTTED VALUES #
    #t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2], is_diminution=True)
    #assert t.lilypond_format == "\\fraction \\times 3/5 {\n\tc'16\n\tc'8\n\tc'8\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2, 3], is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 3/4 {\n\tc'32\n\tc'16\n\tc'16\n\tc'16.\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2, 3, 3], is_diminution=True)
    assert t.lilypond_format == "\\fraction \\times 6/11 {\n\tc'32\n\tc'16\n\tc'16\n\tc'16.\n\tc'16.\n}"

    t = tuplettools.leaf_to_tuplet_with_ratio(note, [1, 2, 2, 3, 3, 4], is_diminution=True)
    assert t.lilypond_format == "\\times 4/5 {\n\tc'64\n\tc'32\n\tc'32\n\tc'32.\n\tc'32.\n\tc'16\n}"



import doctest
import importlib
import os
import pytest
from abjad.tools import iotools
from abjad.tools.developerscripttools.DeveloperScript import DeveloperScript


class TestAndRebuildScript(DeveloperScript):

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def alias(self):
        return 're'

    @property
    def long_description(self):
        return None

    @property
    def scripting_group(self):
        return None

    @property
    def short_description(self):
        return 'Run py.test -x, doctest -x and then rebuild the API.'

    @property
    def version(self):
        return 1.0

    ### PUBLIC PROPERTIES ###

    def get_terminal_width(self):
        """Borrowed from the py lib."""
        try:
            import termios, fcntl, struct
            call = fcntl.ioctl(0, termios.TIOCGWINSZ,
                               struct.pack('hhhh', 0, 0, 0, 0))
            height, width = struct.unpack('hhhh', call)[:2]
            terminal_width = width
        except (SystemExit, KeyboardInterrupt):
            raise
        except:
            # FALLBACK
            terminal_width = int(os.environ.get('COLUMNS', 80)) - 1
        return terminal_width

    def process_args(self, args):
        iotools.clear_terminal()
        if not self.run_doctest(args):
            print
            if not self.run_pytest(args):
                print
                self.rebuild_docs(args)

    def rebuild_docs(self, args):
        from abjad.tools import developerscripttools
        developerscripttools.BuildApiScript()(['-X', '-M'])

    def run_doctest(self, args):
        start_message = ' doctest session starts '
        stop_message = ' {} of {} tests passed, in {} modules '

        print start_message.center(self.get_terminal_width(), '=')
        print

        globs = importlib.import_module('abjad').__dict__.copy()
        try:
            globs.update(importlib.import_module('experimental.tools').__dict__)
            globs.update(importlib.import_module('experimental.demos').__dict__)
        except ImportError:
            pass
        failed_tests = 0
        passed_tests = 0
        passed_modules = 0
        optionflags = doctest.NORMALIZE_WHITESPACE|doctest.ELLIPSIS|doctest.REPORT_NDIFF|doctest.REPORT_ONLY_FIRST_FAILURE
        for dir_path, dir_names, file_names in os.walk('.'):
            for file_name in file_names:
                if file_name.endswith('.py') and \
                    not file_name.startswith('test_') and \
                    not file_name == '__init__.py':
                    full_file_name = os.path.abspath(os.path.join(dir_path, file_name))
                    print os.path.relpath(full_file_name)
                    failure_count, test_count = doctest.testfile(
                        full_file_name,
                        module_relative=False,
                        globs=globs,
                        optionflags=optionflags
                        )
                    if failure_count:
                        failed_tests += failure_count
                        return True
                    passed_modules += 1
                    passed_tests += test_count

        print
        print stop_message.format(
            passed_tests - failed_tests,
            passed_tests,
            passed_modules).center(self.get_terminal_width(), '=')

        return False

    def run_pytest(self, args):
        options = ['-x', '-rf', '.']
        return pytest.main(options)        

    def setup_argument_parser(self, parser):
        pass



import copy
import abc
from abjad.tools import componenttools
from abjad.tools import durationtools
from abjad.tools import leaftools
from abjad.tools import lilypondproxytools
from abjad.tools import timespantools
from abjad.tools.abctools import AbjadObject


class Spanner(AbjadObject):
    '''.. versionadded:: 1.1

    Any type of notation object that stretches horizontally
    and encompasses some number of notes, rest, chords, tuplets,
    measures, voices or other Abjad components.

    Beams, slurs, hairpins, trills, glissandi and piano pedal brackets
    all stretch horizontally on the page to encompass multiple notes
    and all implement as Abjad spanners.
    That is, these spanner all have an obvious graphic reality with
    definite start-, stop- and midpoints.

    Abjad also implements a number of spanners of a different type,
    such as tempo and instrument spanners, which mark a group of notes,
    rests, chords or measues as carrying a certain tempo or being
    played by a certain instrument.

    The spanner class described here
    abstracts the functionality that all such spanners, both graphic
    and nongraphics, share.
    This shared functionality includes methods to add, remove, inspect
    and test components governed by the spanner, as well as basic
    formatting properties.
    The other spanner classes, such as beam and glissando, all inherit from
    this class and receive the functionality implemented here.
    '''

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self, components=None):
        self._components = []
        self._contiguity_constraint = 'thread'
        self._initialize_components(components)

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        r'''.. versionadded:: 2.9

        Call spanner on `expr` as a shortcut to extend spanner::

            >>> staff = Staff("c'8 d'8 e'8 f'8")

        ::

            >>> beam = beamtools.BeamSpanner()
            >>> beam(staff[:])
            BeamSpanner(c'8, d'8, e'8, f'8)

        ::

            >>> f(staff)
            \new Staff {
                c'8 [
                d'8
                e'8
                f'8 ]
            }

        The method is provided as an experimental way of unifying 
        spanner and mark attachment syntax.

        Return spanner.
        '''
        self.extend(expr)
        return self

    def __contains__(self, expr):
        for x in self._components:
            if x is expr:
                return True
        else:
            return False

    def __copy__(self, *args):
        new = type(self)(*self.__getnewargs__())
        if getattr(self, '_override', None) is not None:
            new._override = copy.copy(self.override)
        if getattr(self, '_set', None) is not None:
            new._set = copy.copy(self.set)
        self._copy_keyword_args(new)
        return new

    __deepcopy__ = __copy__

    def __getitem__(self, expr):
        return self._components.__getitem__(expr)

    def __getnewargs__(self):
        return ()

    def __len__(self):
        return self._components.__len__()

    def __lt__(self, other):
        '''Trivial comparison to allow doctests to work.
        '''
        if not isinstance(other, Spanner):
            raise TypeError
        return repr(self) < repr(other)

    def __repr__(self):
        return '%s(%s)' % (type(self).__name__, self._compact_summary)

    ### READ-ONLY PRIVATE PROPERTIES ###

    @property
    def _compact_summary(self):
        len_self = len(self)
        if not len_self:
            return ''
        elif 0 < len_self <= 8:
            return ', '.join([x._compact_representation for x in self])
        else:
            left = ', '.join([x._compact_representation for x in self[:2]])
            right = ', '.join([x._compact_representation for x in self[-2:]])
            number_in_middle = len_self - 4
            middle = ', ... [%s] ..., ' % number_in_middle
            return left + middle + right

    @property
    def _summary(self):
        if 0 < len(self):
            return ', '.join([str(x) for x in self])
        else:
            return ' '

    ### PRIVATE METHODS ###

    def _block_all_components(self):
        '''Not composer-safe.
        '''
        for component in self:
            self._block_component(component)

    def _block_component(self, component):
        '''Not composer-safe.
        '''
        component._spanners.remove(self)

    def _copy(self, components):
        '''Return copy of spanner with components.
        Components must be an iterable of components already contained in spanner.
        '''
        my_components = self._components[:]
        self._components = []
        result = copy.deepcopy(self)
        self._components = my_components
        for component in components:
            assert component in self
        for component in components:
            result._components.append(component)
        result._unblock_all_components()
        return result

    @abc.abstractmethod
    def _copy_keyword_args(self, new):
        raise NotImplemented

    def _duration_offset_in_me(self, leaf):
        return leaf.start_offset - self.start_offset

    def _format_after_leaf(self, leaf):
        result = []
        if self._is_my_last_leaf(leaf):
            result.extend(getattr(self, '_reverts', []))
        return result

    def _format_before_leaf(self, leaf):
        result = []
        if self._is_my_first_leaf(leaf):
            result.extend(getattr(self, '_overrides', []))
        return result

    def _format_right_of_leaf(self, leaf):
        result = []
        return result

    def _fracture_left(self, i):
        left = self._copy(self[:i])
        right = self._copy(self[i:])
        self._block_all_components()
        return self, left, right

    def _fracture_right(self, i):
        left = self._copy(self[:i+1])
        right = self._copy(self[i+1:])
        self._block_all_components()
        return self, left, right

    def _fuse_by_reference(self, spanner):
        result = self._copy(self[:])
        result.extend(spanner.components)
        self._block_all_components()
        spanner._block_all_components()
        return [(self, spanner, result)]

    def _initialize_components(self, components):
        from abjad.tools import iterationtools
        if isinstance(components, componenttools.Component):
            components = [components]
        elif not components:
            components = []
        # TODO: Author staff-level contiguity check in tools/check.
        #       Include optional staff-level contiguity check here.
        if self._contiguity_constraint == 'thread':
            leaves = list(iterationtools.iterate_leaves_in_expr(components))
            assert componenttools.all_are_thread_contiguous_components(leaves)
        self.extend(components)

    def _insert(self, i, component):
        '''Insert component in spanner at index i.
        Not composer-safe and may mangle spanners.
        '''
        component._spanners.add(self)
        self._components.insert(i, component)

    # TODO: author tests.
    def _is_exterior_leaf(self, leaf):
        '''True if leaf is first or last in spanner.
        True if next leaf or prev leaf is none.
        False otherwise.
        '''
        if self._is_my_first_leaf(leaf):
            return True
        elif self._is_my_last_leaf(leaf):
            return True
        elif not leaftools.get_nth_leaf_in_thread_from_leaf(leaf, -1) and \
            not leaftools.get_nth_leaf_in_thread_from_leaf(leaf, 1):
            return True
        else:
            return False

    def _is_my_first(self, leaf, klass):
        from abjad.tools import spannertools
        for component in spannertools.iterate_components_in_spanner(self, klass=klass):
            if component is leaf:
                return True
            else:
                return False

    def _is_my_first_leaf(self, leaf):
        from abjad.tools import spannertools
        try:
            first_leaf = spannertools.get_nth_leaf_in_spanner(self, 0)
            return leaf is first_leaf
        except IndexError:
            return False

    def _is_my_last(self, leaf, klass):
        from abjad.tools import spannertools
        components = spannertools.iterate_components_in_spanner(self, klass=klass, reverse=True)
        for component in components:
            if component is leaf:
                return True
            else:
                return False

    def _is_my_last_leaf(self, leaf):
        from abjad.tools import spannertools
        try:
            last_leaf = spannertools.get_nth_leaf_in_spanner(self, -1)
            return leaf is last_leaf
        except IndexError:
            return False

    def _is_my_only(self, leaf, klass):
        from abjad.tools import spannertools
        i, components = None, spannertools.iterate_components_in_spanner(self, klass=klass)
        for i, component in enumerate(components):
            if 0 < i:
                return False
        return i == 0

    def _is_my_only_leaf(self, leaf):
        return self._is_my_first_leaf(leaf) and self._is_my_last_leaf(leaf)

    def _remove(self, component):
        '''Remove 'component' from spanner.
        Remove spanner from component's aggregator.
        Not composer-safe and may leave discontiguous spanners.
        '''
        self._sever_component(component)

    def _remove_component(self, component):
        '''Not composer-safe.
        '''
        for i, x in enumerate(self._components):
            if x is component:
                self._components.pop(i)
                break
        else:
            raise ValueError('component "{}" not in spanner components list.'.format(component))

    def _reverse_components(self):
        r'''Reverse order of spanner components.

        Not composer-safe because reversing the order of spanner components
        could scramble components of some other spanner.

        Call method only as part of a full component- and spanner-reversal routine.

        Spanner subclasses with mapping variables (like the 'durations' list
        attaching to durated complex beam spanners) should override this method
        to reverse mapping elements.
        '''
        self._components.reverse()

    def _sever_all_components(self):
        '''Not composer-safe.
        '''
        for n in reversed(range(len(self))):
            component = self[n]
            self._sever_component(component)

    def _sever_component(self, component):
        '''Not composer-safe.
        '''
        self._block_component(component)
        self._remove_component(component)

    def _unblock_all_components(self):
        '''Not composer-safe.
        '''
        for component in self:
            self._unblock_component(component)

    def _unblock_component(self, component):
        '''Not composer-safe.
        '''
        component._spanners.add(self)

    ### PUBLIC PROPERTIES ###

    @property
    def components(self):
        '''Return read-only tuple of components in spanner. ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[:2])
            >>> spanner.components
            (Note("c'8"), Note("d'8"))

        .. versionchanged:: 1.1
            Now returns an (immutable) tuple instead of a (mutable) list.
        '''
        return tuple(self._components[:])

    @property
    def duration_in_seconds(self):
        '''Sum of duration of all leaves in spanner, in seconds.'''
        duration = durationtools.Duration(0)
        for leaf in self.leaves:
            duration += leaf.duration_in_seconds
        return duration

    @property
    def leaves(self):
        '''Return read-only tuple of leaves in spanner. ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[:2])
            >>> spanner.leaves
            (Note("c'8"), Note("d'8"))

        .. note:: When dealing with large, complex scores accessing
            this attribute can take some time. Best to make a local
            copy with leaves = spanner.leaves first. Or use spanner-
            specific iteration tools.
        '''
        from abjad.tools import iterationtools
        result = []
        for component in self._components:
            # EXPERIMENTAL: expand to allow staff-level spanner eventually #
            for node in iterationtools.iterate_components_depth_first(component):
                if isinstance(node, leaftools.Leaf):
                    result.append(node)
        result = tuple(result)
        return result

    @property
    def override(self):
        '''LilyPond grob override component plug-in.
        '''
        if not hasattr(self, '_override'):
            self._override = lilypondproxytools.LilyPondGrobOverrideComponentPlugIn()
        return self._override

    @property
    def preprolated_duration(self):
        '''Sum of preprolated duration of all components in spanner.
        '''
        return sum([component.preprolated_duration for component in self])

    @property
    def prolated_duration(self):
        '''Sum of prolated duration of all components in spanner.
        '''
        return sum([component.prolated_duration for component in self])

    @property
    def set(self):
        '''LilyPond context setting component plug-in.
        '''
        if not hasattr(self, '_set'):
            self._set = lilypondproxytools.LilyPondContextSettingComponentPlugIn()
        return self._set

    @property
    def start_offset(self):
        '''Read-only start offset of spanner.
        '''
        if len(self):
            return self[0].start_offset
        else:
            return Duration(0)

    @property
    def stop_offset(self):
        '''Read-only stop offset of spanner.
        '''
        if len(self):
            return self[-1].stop_offset
        else:
            return Duration(0)

    @property
    def timespan(self):
        '''Read-only timespan of spanner.
        '''
        return timespantools.Timespan(start_offset=self.start_offset, stop_offset=self.stop_offset)

    @property
    def written_duration(self):
        '''Sum of written duration of all components in spanner.
        '''
        return sum([component.written_duration for component in self])

    ### PUBLIC METHODS ###

    def append(self, component):
        '''Add `component` to right of spanner.

        ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[:2])
            >>> spanner
            BeamSpanner(c'8, d'8)

        ::

            >>> spanner.append(voice[2])
            >>> spanner
            BeamSpanner(c'8, d'8, e'8)

        Return none.
        '''
        if self._contiguity_constraint == 'thread':
            components = self[-1:] + [component]
            assert componenttools.all_are_thread_contiguous_components(components), repr(components)
        component._spanners.add(self)
        self._components.append(component)

    def append_left(self, component):
        '''Add `component` to left of spanner.

        ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[2:])
            >>> spanner
            BeamSpanner(e'8, f'8)

        ::

            >>> spanner.append_left(voice[1])
            >>> spanner
            BeamSpanner(d'8, e'8, f'8)

        Return none.
        '''
        components = [component] + self[:1]
        assert componenttools.all_are_thread_contiguous_components(components)
        component._spanners.add(self)
        self._components.insert(0, component)

    def clear(self):
        r'''Remove all components from spanner::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[:])
            >>> spanner
            BeamSpanner(c'8, d'8, e'8, f'8)

        ::

            >>> spanner.clear()
            >>> spanner
            BeamSpanner()

        Return none.
        '''
        self._sever_all_components()

    def extend(self, components):
        '''Add iterable `components` to right of spanner::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[:2])
            >>> spanner
            BeamSpanner(c'8, d'8)

        ::

            >>> spanner.extend(voice[2:])
            >>> spanner
            BeamSpanner(c'8, d'8, e'8, f'8)

        Return none.
        '''
        component_input = self[-1:]
        component_input.extend(components)
        if self._contiguity_constraint == 'thread':
            assert componenttools.all_are_thread_contiguous_components(component_input)
        for component in components:
            self.append(component)

    def extend_left(self, components):
        '''Add iterable `components` to left of spanner::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[2:])
            >>> spanner
            BeamSpanner(e'8, f'8)

        ::

            >>> spanner.extend_left(voice[:2])
            >>> spanner
            BeamSpanner(c'8, d'8, e'8, f'8)

        Return none.
        '''
        component_input = components + self[:1]
        assert componenttools.all_are_thread_contiguous_components(component_input)
        for component in reversed(components):
            self.append_left(component)

    def fracture(self, i, direction=None):
        r'''Fracture spanner at `direction` of component at index `i`.

        Valid values for `direction` are ``Left``, ``Right`` and ``None``.

        Return original, left and right spanners. ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> beam = beamtools.BeamSpanner(voice[:])
            >>> beam
            BeamSpanner(c'8, d'8, e'8, f'8)

        ::

            >>> f(voice)
            \new Voice {
                c'8 [
                d'8
                e'8
                f'8 ]
            }

        ::

            >>> show(voice) # doctest: +SKIP

        ::

            >>> beam.fracture(1, direction=Left)
            (BeamSpanner(c'8, d'8, e'8, f'8), BeamSpanner(c'8), BeamSpanner(d'8, e'8, f'8))

        ::

            >>> f(voice)
            \new Voice {
                c'8 [ ]
                d'8 [
                e'8
                f'8 ]
            }

        ::

            >>> show(voice) # doctest: +SKIP

        Set `direction=None` to fracture on both left and right sides.

        Return tuple.
        '''
        if i < 0:
            i = len(self) + i
        if direction == Left:
            return self._fracture_left(i)
        elif direction == Right:
            return self._fracture_right(i)
        elif direction is None:
            left = self._copy(self[:i])
            right = self._copy(self[i+1:])
            center = self._copy(self[i:i+1])
            self._block_all_components()
            return self, left, center, right
        else:
            raise ValueError('direction {!r} must be Left, Right or None.'.format(direction))

    def fuse(self, spanner):
        r'''Fuse contiguous spanners.

        Return new spanner. ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> left_beam = beamtools.BeamSpanner(voice[:2])
            >>> right_beam = beamtools.BeamSpanner(voice[2:])

        ::

            >>> print voice.lilypond_format
            \new Voice {
                c'8 [
                d'8 ]
                e'8 [
                f'8 ]
            }

        ::

            >>> left_beam.fuse(right_beam)
            [(BeamSpanner(c'8, d'8), BeamSpanner(e'8, f'8), BeamSpanner(c'8, d'8, e'8, f'8))]

        ::

            >>> print voice.lilypond_format
            \new Voice {
                c'8 [
                d'8
                e'8
                f'8 ]
            }

        Return list.
        '''
        return self._fuse_by_reference(spanner)

    def index(self, component):
        '''Return nonnegative integer index of `component` in spanner::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = beamtools.BeamSpanner(voice[2:])
            >>> spanner
            BeamSpanner(e'8, f'8)

        ::

            >>> spanner.index(voice[-2])
            0

        Return nonnegative integer.
        '''
        for i, x in enumerate(self._components):
            if x is component:
                return i
        else:
            raise IndexError

    def pop(self):
        r'''Remove and return rightmost component in spanner:

        ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.SlurSpanner(voice[:])
            >>> spanner
            SlurSpanner(c'8, d'8, e'8, f'8)

        ::

            >>> f(voice)
            \new Voice {
                c'8 (
                d'8
                e'8
                f'8 )
            }

        ::

            >>> show(voice) # doctest: +SKIP

        ::

            >>> spanner.pop()
            Note("f'8")

        ::

            >>> spanner
            SlurSpanner(c'8, d'8, e'8)

        ::

            >>> f(voice)
            \new Voice {
                c'8 (
                d'8
                e'8 )
                f'8
            }

        ::

            >>> show(voice) # doctest: +SKIP

        Return component.
        '''
        component = self[-1]
        self._sever_component(component)
        return component

    def pop_left(self):
        r'''Remove and return leftmost component in spanner:

        ::

            >>> voice = Voice("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.SlurSpanner(voice[:])
            >>> spanner
            SlurSpanner(c'8, d'8, e'8, f'8)

        ::

            >>> f(voice)
            \new Voice {
                c'8 (
                d'8
                e'8
                f'8 )
            }

        ::

            >>> show(voice) # doctest: +SKIP

        ::

            >>> spanner.pop_left()
            Note("c'8")

        ::

            >>> spanner
            SlurSpanner(d'8, e'8, f'8)

        ::

            >>> f(voice)
            \new Voice {
                c'8
                d'8 (
                e'8
                f'8 )
            }

        ::

            >>> show(voice) # doctest: +SKIP

        Return component.
        '''
        component = self[0]
        self._sever_component(component)
        return component



from abjad.tools.abctools.AbjadObject import AbjadObject
from abjad.tools.scoretools.PerformerInventory import PerformerInventory


class InstrumentationSpecifier(AbjadObject):
    r'''.. versionadded:: 2.5

    Abjad model of score instrumentation::

        >>> flute = scoretools.Performer('Flute')
        >>> flute.instruments.append(instrumenttools.Flute())
        >>> flute.instruments.append(instrumenttools.AltoFlute())

    ::

        >>> guitar = scoretools.Performer('Guitar')
        >>> guitar.instruments.append(instrumenttools.Guitar())

    ::

        >>> instrumentation_specifier = scoretools.InstrumentationSpecifier([flute, guitar])

    ::

        >>> z(instrumentation_specifier)
        scoretools.InstrumentationSpecifier(
            performers=scoretools.PerformerInventory([
                scoretools.Performer(
                    name='Flute',
                    instruments=instrumenttools.InstrumentInventory([
                        instrumenttools.Flute(),
                        instrumenttools.AltoFlute()
                        ])
                    ),
                scoretools.Performer(
                    name='Guitar',
                    instruments=instrumenttools.InstrumentInventory([
                        instrumenttools.Guitar()
                        ])
                    )
                ])
            )

    Return instrumentation specifier.
    '''

    def __init__(self, performers=None):
        self._performers = PerformerInventory()
        self.performers = performers

    ### SPECIAL METHODS ###
    
    def __eq__(self, other):
        if isinstance(other, type(self)):
            # TODO: implement perform sort
            #if sorted(self.performers) == sorted(other.performers):
            if self.performers == other.performers:
                return True
        return False

    def __ne__(self, other):
        return not self == other

    ### PUBLIC PROPERTIES ###

    @property
    def instrument_count(self):
        r'''Read-only number of instruments in score::
    
            >>> instrumentation_specifier.instrument_count
            3

        Return nonnegative integer.
        '''
        return len(self.instruments)

    @property
    def instruments(self):
        r'''Read-only list of instruments derived from performers::

            >>> instrumentation_specifier.instruments
            [Flute(), AltoFlute(), Guitar()]

        Return list.
        '''        
        instruments = []
        for performer in self.performers:
            instruments.extend(performer.instruments)
        return instruments

    @property
    def performer_count(self):
        r'''Read-only number of performers in score::

            >>> instrumentation_specifier.performer_count
            2

        Return nonnegative integer.
        '''
        return len(self.performers)

    @property
    def performer_name_string(self):
        r'''Read-only string of performer names::

            >>> instrumentation_specifier.performer_name_string
            'Flute, Guitar'

        Return string.
        '''
        if self.performers:
            return ', '.join([performer.name for performer in self.performers])
        else:
            return ''

    @apply
    def performers():
        def fget(self):
            r'''Read / write list of performers in score::

                >>> z(instrumentation_specifier.performers)
                scoretools.PerformerInventory([
                    scoretools.Performer(
                        name='Flute',
                        instruments=instrumenttools.InstrumentInventory([
                            instrumenttools.Flute(),
                            instrumenttools.AltoFlute()
                            ])
                        ),
                    scoretools.Performer(
                        name='Guitar',
                        instruments=instrumenttools.InstrumentInventory([
                            instrumenttools.Guitar()
                            ])
                        )
                    ])

            Return performer inventory.
            '''
            return self._performers
        def fset(self, performers):
            from abjad.tools import scoretools
            assert isinstance(performers, (list, type(None)))
            if performers is None:
                self._performers[:] = []
            else:
                assert all([isinstance(x, scoretools.Performer) for x in performers])
                self._performers[:] = list(performers[:])
        return property(**locals())



import types
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTInheritanceDiagram(ReSTDirective):
    '''An ReST inheritance diagram directive:

    ::

        >>> documentationtools.ReSTInheritanceDiagram(argument=beamtools.BeamSpanner)
        ReSTInheritanceDiagram(
            argument='abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner',
            options={
                'private-bases': True
                }
            )

    ::

        >>> print _.rest_format
        .. inheritance-diagram:: abjad.tools.beamtools.BeamSpanner.BeamSpanner.BeamSpanner
           :private-bases:

    Return `ReSTInheritanceDiagram` instance.
    '''

    ### INITIALIZER ###

    def __init__(self, argument=None, children=None, name=None, options=None):
        if isinstance(argument, types.TypeType):
            argument = argument.__module__ + '.' + argument.__name__
        new_options = {'private-bases': True}
        if options is not None:
            new_options.update(options)
        ReSTDirective.__init__(self, argument=argument, children=children, name=name, options=new_options)

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def directive(self):
        return 'inheritance-diagram'



from abjad.tools import durationtools


def calculate_sustain_centroid_of_intervals(intervals):
    '''Return a weighted mean, such that the centroid of each interval
    in `intervals` are the values, and the weights are their durations.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return None

    weighted_centroids = sum([(x.center * x.duration) for x in tree])
    sum_of_weights = sum([x.duration for x in tree])
    return durationtools.Offset(weighted_centroids) / sum_of_weights



from abjad.tools.quantizationtools.AttackPointOptimizer import AttackPointOptimizer

class NullAttackPointOptimizer(AttackPointOptimizer):
    '''Concrete ``AttackPointOptimizer`` subclass which performs
    no attack point optimization.

    Return ``NullAttackPointOptimizer`` instance.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        pass



from abjad.tools import configurationtools
import os


def show(expr, return_timing=False, suppress_pdf=False, docs=False):
    '''Show `expr`::

        >>> note = Note("c'4")
        >>> show(note) # doctest: +SKIP

    Show `expr` and return both Abjad and LilyPond processing time in seconds::

        >>> staff = Staff(Note("c'4") * 200)
        >>> show(note, return_timing=True) # doctest: +SKIP
        (0, 3)

    Wrap `expr` in a LilyPond file with settings and overrides suitable
    for the Abjad reference manual When `docs` is true. 

    Return none or timing tuple.

    Abjad writes LilyPond input files to the ``~/.abjad/output`` directory by default.

    You may change this by setting the ``abjad_output`` variable in the ``config.py`` file.
    '''
    from abjad import ABJCFG
    from abjad.tools.iotools._log_render_lilypond_input import _log_render_lilypond_input
    from abjad.tools.iotools._open_file import _open_file

    name, actual_format_time, actual_lily_time = _log_render_lilypond_input(expr, docs=docs)

    # do not open PDF if we're running py.test regression battery
    if not suppress_pdf:
        pdf_viewer = ABJCFG['pdf_viewer']
        ABJADOUTPUT = ABJCFG['abjad_output']
        name = os.path.join(ABJADOUTPUT, name)
        _open_file('%s.pdf' % name[:-3], pdf_viewer)

    # return timing if requested
    if return_timing:
        return actual_format_time, actual_lily_time



from abjad import *


def test_InstrumentInventory_storage_format_01():

    inventory = instrumenttools.InstrumentInventory([
        instrumenttools.Flute(),
        instrumenttools.Violin()])

    r'''
    instrumenttools.InstrumentInventory([
        instrumenttools.Flute(),
        instrumenttools.Violin()
        ])
    '''

    assert inventory.storage_format == 'instrumenttools.InstrumentInventory([\n\tinstrumenttools.Flute(),\n\tinstrumenttools.Violin()\n\t])'



import abc
from abjad.tools.pitchtools.HarmonicObject import HarmonicObject
from abjad.tools.pitchtools.IntervalObjectClass import IntervalObjectClass


class HarmonicIntervalClassObject(IntervalObjectClass, HarmonicObject):
    '''.. versionadded:: 2.0

    Harmonic interval-class base class.
    '''

    ### CLASS ATTRIBUTES ###

    __metaclass__ = abc.ABCMeta

    ### INITIALIZER ###

    @abc.abstractmethod
    def __init__(self):
        pass



from abjad.tools import iterationtools
from abjad.tools import layouttools


def configure_multiple_voice_rhythmic_staves(lilypond_file):
    '''Configure multiple-voice rhythmic staves in `lilypond_file`.

    Operate in place and return none.
    '''

    for staff in iterationtools.iterate_staves_in_expr(lilypond_file.score_block[0]):
        if staff.is_parallel:
            assert len(staff) == 2
            voice_1 = staff[0]
            voice_1.override.note_head.Y_offset = 0.5
            voice_1.override.stem.direction = Up
            voice_2 = staff[1]
            voice_2.override.note_head.Y_offset = -0.5
            voice_2.override.stem.direction = Down 
            spacing_vector = layouttools.make_spacing_vector(0, 0, 6, 0)
            staff.override.vertical_axis_group.staff_staff_spacing = spacing_vector



from abjad.tools import contexttools
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools.measuretools.Measure.Measure import Measure


class DynamicMeasure(Measure):
    r'''.. versionadded:: 1.1

    Measure sets time signature dynamically to exactly equal contents duration::

        >>> measure = measuretools.DynamicMeasure("c'8 d'8 e'8")

    ::

        >>> measure
        DynamicMeasure(3/8, [c'8, d'8, e'8])

    ::

        >>> f(measure)
        {
            \time 3/8
            c'8
            d'8
            e'8
        }

    Return dynamic measure.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('_denominator', '_time_signature_is_current', '_suppress_time_signature', )

    ### INITIALIZER ###

    def __init__(self, music=None, **kwargs):
        Measure.__init__(self, time_signature=(99, 99), music=music, **kwargs)
        self._denominator = None
        self._time_signature_is_current = False
        self._suppress_time_signature = False
        self._update_time_signature()

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        return ()

    ### PRIVATE METHODS ###

    def _update_time_signature(self):
        if self.denominator:
            time_signature_pair = mathtools.NonreducedFraction(self.contents_duration)
            time_signature_pair = time_signature_pair.with_denominator(self.denominator)
        else:
            time_signature_pair = (self.contents_duration.numerator, self.contents_duration.denominator)
        time_signature = contexttools.TimeSignatureMark(time_signature_pair)
        time_signature.suppress = self.suppress_time_signature
        contexttools.detach_time_signature_marks_attached_to_component(self)
        time_signature.attach(self)
        self._time_signature_is_current = True

    ### PUBLIC PROPERTIES ###

    @apply
    def denominator():
        def fget(self):
            r'''Get explicit denominator of dynamic measure::

                >>> measure = measuretools.DynamicMeasure("c'8 d'8 e'8 f'8")

            ::

                >>> measure.denominator is None
                True

            Set explicit denominator of dynamic measure::

                >>> measure.denominator = 8

            ::

                >>> f(measure)
                {
                    \time 4/8
                    c'8
                    d'8
                    e'8
                    f'8
                }

            Set positive integer or none.
            '''
            return self._denominator
        def fset(self, arg):
            assert isinstance(arg, (int, long, type(None)))
            self._denominator = arg
            self._update_time_signature()
        return property(**locals())

    @property
    def preprolated_duration(self):
        return self.contents_duration

    @apply
    def suppress_time_signature():
        def fget(self):
            r'''Get time signature suppression indicator::

            >>> measure = measuretools.DynamicMeasure("c'8 d'8 e'8 f'8")

        ::

            >>> f(measure)
            {
                \time 1/2
                c'8
                d'8
                e'8
                f'8
            }

        ::

            >>> measure.suppress_time_signature
            False

        Set time signature suppression indicator::

            >>> measure.suppress_time_signature = True

        ::

            >>> measure.suppress_time_signature
            True

        ::

            >>> f(measure)
            {
                c'8
                d'8
                e'8
                f'8
            }

        Set boolean.
        '''
            return self._suppress_time_signature
        def fset(self, arg):
            assert isinstance(arg, (bool, type(None)))
            self._suppress_time_signature = arg
            self._update_time_signature()
        return property(**locals())

    ### PUBLIC METHODS ###

    def extend(self, expr):
        r'''Extend dynamic measure::

            >>> measure = measuretools.DynamicMeasure("c'8 d'8 e'8")

        ::

            >>> f(measure)
            {
                \time 3/8
                c'8
                d'8
                e'8
            }

        ::

            >>> measure.extend([Note("f'8"), Note("g'8")])

        ::

            >>> f(measure)
            {
                \time 5/8
                c'8
                d'8
                e'8
                f'8
                g'8
            }

        Return none.
        '''
        Measure.extend(self, expr)
        self._update_time_signature()



def fuse_tangent_or_overlapping_intervals(intervals):
    '''Fuse all tangent or overlapping intervals and return an `TimeIntervalTree` of the result ::

        >>> from abjad.tools.timeintervaltools import TimeInterval
        >>> from abjad.tools.timeintervaltools import TimeIntervalTree

    ::

        >>> a = TimeInterval(0, 10)
        >>> b = TimeInterval(5, 15)
        >>> c = TimeInterval(15, 25)
        >>> tree = TimeIntervalTree([a, b, c])
        >>> timeintervaltools.fuse_tangent_or_overlapping_intervals(tree)
        TimeIntervalTree([
            TimeInterval(Offset(0, 1), Offset(25, 1), {})
        ])

    Return TimeIntervalTree.
    '''
    from abjad.tools import timeintervaltools

    tree = timeintervaltools.TimeIntervalTree(intervals)

    if not tree:
        return tree

    trees = [timeintervaltools.TimeIntervalTree(group) for group in \
        timeintervaltools.group_tangent_or_overlapping_intervals_and_yield_groups(tree)]

    return timeintervaltools.TimeIntervalTree([
        timeintervaltools.TimeInterval(tree.earliest_start, tree.latest_stop) for tree in trees
    ])



from abjad.tools.abctools import AbjadObject


class LilyPondDuration(AbjadObject):
    '''Model of a duration in LilyPond.

    Not composer-safe.

    Used internally by LilyPondParser.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ('duration', 'multiplier')

    ### INITIALIZER ###

    def __init__(self, duration, multiplier=None):
        self.duration = duration
        self.multiplier = multiplier



from abjad.tools.rhythmmakertools.DivisionIncisedRestRhythmMaker import DivisionIncisedRestRhythmMaker


class RestRhythmMaker(DivisionIncisedRestRhythmMaker):
    r'''.. versionadded:: 2.8

    Rest rhythm-maker::

        >>> maker = rhythmmakertools.RestRhythmMaker()

    ::

        >>> divisions = [(5, 16), (3, 8)]
        >>> leaf_lists = maker(divisions)
        >>> leaves = sequencetools.flatten_sequence(leaf_lists)

    ::

        >>> measures = measuretools.make_measures_with_full_measure_spacer_skips(divisions)
        >>> staff = Staff(measures)
        >>> measures = measuretools.replace_contents_of_measures_in_expr(staff, leaves)

    ::

        >>> f(staff)
        \new Staff {
            {
                \time 5/16
                r4
                r16
            }
            {
                \time 3/8
                r4.
            }
        }

    Usage follows the two-step instantiate-then-call pattern shown here.

    Return rhythm-maker.
    '''

    ### CLASS ATTRIBUTES ###

    _default_positional_input_arguments = ()

    ### INITIALIZER ###

    def __init__(self):
        DivisionIncisedRestRhythmMaker.__init__(
            self, [], [0], [], [0], 1,
            decrease_durations_monotonically=True, tie_rests=False
            )



def repeat_leaves_in_expr(expr, total=1):
    r'''.. versionadded:: 1.1

    Repeat leaves in `expr` and extend spanners::

        >>> staff = Staff("c'8 d'8 e'8 f'8")
        >>> beamtools.BeamSpanner(staff.leaves)
        BeamSpanner(c'8, d'8, e'8, f'8)
        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            f'8 ]
        }

    ::

        >>> result = leaftools.repeat_leaves_in_expr(staff[2:], total=3)

    ::

        >>> f(staff)
        \new Staff {
            c'8 [
            d'8
            e'8
            e'8
            e'8
            f'8
            f'8
            f'8 ]
        }

    Preserve leaf written durations.

    Preserve parentage and spanners.

    Return none.

    .. versionchanged:: 2.0
        renamed ``leaftools.multiply()`` to
        ``leaftools.repeat_leaves_in_expr()``.
    '''
    from abjad.tools import iterationtools
    from abjad.tools import leaftools

    for leaf in iterationtools.iterate_leaves_in_expr(expr, reverse=True):
        leaftools.repeat_leaf(leaf, total)



from abjad import *


def test_markuptools_make_vertically_adjusted_composer_markup_01():

    markup = markuptools.make_vertically_adjusted_composer_markup('Josquin Desprez')

    r'''
    \markup {
        \override
            #'(font-name . "Times")
            {
                \hspace
                    #0
                \raise
                    #-20
                    \fontsize
                        #3
                        "Josquin Desprez"
                \hspace
                    #0
            }
        }
    '''

    assert markup.lilypond_format == '\\markup { \\override #\'(font-name . "Times") { \\hspace #0 \\raise #-20 \\fontsize #3 "Josquin Desprez" \\hspace #0 } }'
    assert markup.indented_lilypond_format == '\\markup {\n\t\\override\n\t\t#\'(font-name . "Times")\n\t\t{\n\t\t\t\\hspace\n\t\t\t\t#0\n\t\t\t\\raise\n\t\t\t\t#-20\n\t\t\t\t\\fontsize\n\t\t\t\t\t#3\n\t\t\t\t\t"Josquin Desprez"\n\t\t\t\\hspace\n\t\t\t\t#0\n\t\t}\n\t}'



from abjad.tools import contexttools
from abjad.tools import leaftools
from abjad.tools import tietools
from abjad.tools.quantizationtools.AttackPointOptimizer import AttackPointOptimizer


class NaiveAttackPointOptimizer(AttackPointOptimizer):
    '''Concrete ``AttackPointOptimizer`` subclass which optimizes
    attack points by fusing tie leaves within tie chains with
    leaf durations decreasing monotonically.

    ``TieChains`` will be partitioned into sub-``TieChains`` if
    leaves are found with ``TempoMarks`` attached.

    Return ``NaiveAttackPointOptimizer`` instance.
    '''

    ### SPECIAL METHODS ###

    def __call__(self, expr):
        for tie_chain in tietools.iterate_tie_chains_in_expr(expr, reverse=True):

            sub_chains = []
            current_sub_chain = []

            for leaf in tie_chain:
                tempo_marks = contexttools.get_tempo_marks_attached_to_component(leaf)
                if tempo_marks:
                    if current_sub_chain:
                        current_sub_chain = tietools.TieChain(current_sub_chain)
                        sub_chains.append(current_sub_chain)
                    current_sub_chain = []
                current_sub_chain.append(leaf)
            if current_sub_chain:
                current_sub_chain = tietools.TieChain(current_sub_chain)
                sub_chains.append(current_sub_chain)

            for sub_chain in sub_chains:
                leaftools.fuse_leaves_in_tie_chain_by_immediate_parent(sub_chain)




from abjad import *


def test_TieChain_leaves_01():

    staff = Staff("c' ~ c'16")

    assert tietools.get_tie_chain(staff[0]).leaves == tuple(staff[:])


def test_TieChain_leaves_02():

    staff = Staff("c'")

    assert tietools.get_tie_chain(staff[0]).leaves == (staff[0], )



import collections
from abjad.tools import contexttools
from abjad.tools import instrumenttools
from abjad.tools import scoretools
from abjad.tools import stafftools
from abjad.tools import voicetools
from abjad.tools.scoretemplatetools.ScoreTemplate import ScoreTemplate


class GroupedRhythmicStavesScoreTemplate(ScoreTemplate):
    r'''.. versionadded:: 2.9

    Example 1. Grouped rhythmic staves score template with one voice per staff::

        >>> template = scoretemplatetools.GroupedRhythmicStavesScoreTemplate(staff_count=4)
        >>> score = template()

    ::

        >>> score
        Score-"Grouped Rhythmic Staves Score"<<1>>

    ::

        >>> f(score)
        \context Score = "Grouped Rhythmic Staves Score" <<
            \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                \context RhythmicStaff = "Staff 1" {
                    \context Voice = "Voice 1" {
                    }
                }
                \context RhythmicStaff = "Staff 2" {
                    \context Voice = "Voice 2" {
                    }
                }
                \context RhythmicStaff = "Staff 3" {
                    \context Voice = "Voice 3" {
                    }
                }
                \context RhythmicStaff = "Staff 4" {
                    \context Voice = "Voice 4" {
                    }
                }
            >>
        >>

    Example 2. Grouped rhythmic staves score template with more than one voice per staff::

        >>> template = scoretemplatetools.GroupedRhythmicStavesScoreTemplate(staff_count=[2, 1, 2])
        >>> score = template()

    ::

        >>> f(score)
        \context Score = "Grouped Rhythmic Staves Score" <<
            \context StaffGroup = "Grouped Rhythmic Staves Staff Group" <<
                \context RhythmicStaff = "Staff 1" <<
                    \context Voice = "Voice 1-1" {
                    }
                    \context Voice = "Voice 1-2" {
                    }
                >>
                \context RhythmicStaff = "Staff 2" {
                    \context Voice = "Voice 2" {
                    }
                }
                \context RhythmicStaff = "Staff 3" <<
                    \context Voice = "Voice 3-1" {
                    }
                    \context Voice = "Voice 3-2" {
                    }
                >>
            >>
        >>

    Return score template object.   
    '''

    ### INITIALIZER ###

    def __init__(self, staff_count=2):
        assert isinstance(staff_count, (int, list))
        self.context_name_abbreviations = collections.OrderedDict()
        self._staff_count = staff_count

    ### SPECIAL METHODS ###

    def __call__(self):
        staves = []
        if isinstance(self.staff_count, int):
            for index in range(self.staff_count):
                number = index + 1
                voice = voicetools.Voice(name='Voice {}'.format(number))     
                staff = stafftools.RhythmicStaff([voice], name='Staff {}'.format(number))
                staves.append(staff)
                self.context_name_abbreviations['v{}'.format(number)] = voice.name
        elif isinstance(self.staff_count, list):
            for staff_index, voice_count in enumerate(self.staff_count):
                staff_number = staff_index + 1
                staff = stafftools.RhythmicStaff([], name='Staff {}'.format(staff_number))
                assert 1 <= voice_count  
                for voice_index in range(voice_count):
                    voice_number = voice_index + 1
                    if voice_count == 1:
                        voice_identifier = str(staff_number)
                    else:
                        voice_identifier = '{}-{}'.format(staff_number, voice_number)
                        staff.is_parallel = True
                    voice = voicetools.Voice(name='Voice {}'.format(voice_identifier))
                    staff.append(voice)
                    self.context_name_abbreviations['v{}'.format(voice_identifier)] = voice.name
                staves.append(staff)
        
        grouped_rhythmic_staves_staff_group = scoretools.StaffGroup(
            staves, name='Grouped Rhythmic Staves Staff Group')         

        grouped_rhythmic_staves_score = scoretools.Score(
            [grouped_rhythmic_staves_staff_group], name='Grouped Rhythmic Staves Score')

        return grouped_rhythmic_staves_score

    ### READ-ONLY PUBLIC PROPERTIES ###

    @property
    def staff_count(self):
        return self._staff_count



from abjad.tools import componenttools
from abjad.tools import spannertools


def remove_nonfirst_leaves_in_tie_chain(tie_chain):
    r'''Remove nonfirst leaves in `tie_chain`::

        >>> staff = Staff("c'4 ~ c'16")

    ::

        >>> f(staff)
        \new Staff {
            c'4 ~
            c'16
        }

    ::

        >>> tietools.remove_nonfirst_leaves_in_tie_chain(tietools.get_tie_chain(staff[0]))
        TieChain(Note("c'4"),)

    ::

        >>> f(staff)
        \new Staff {
            c'4
        }

    Return `tie_chain`.

    .. versionchanged:: 2.0
        renamed ``tietools.truncate()`` to
        ``tietools.remove_all_leaves_in_tie_chain_except_first()``.

    .. versionchanged:: 2.9
        renamed ``tietools.remove_all_leaves_in_tie_chain_except_first()`` to
        ``tietools.remove_nonfirst_leaves_in_tie_chain()``.
    '''
    from abjad.tools import tietools

    assert isinstance(tie_chain, tietools.TieChain)

    for leaf in tie_chain[1:]:
        componenttools.remove_component_subtree_from_score_and_spanners([leaf])

    first = tie_chain[0]
    spannertools.destroy_spanners_attached_to_component(first, tietools.TieSpanner)

    return tietools.get_tie_chain(first)



from abjad.tools.stafftools.Staff import Staff


class RhythmicStaff(Staff):
    r'''Abjad model of a rhythmic staff:

    ::

        >>> staff = stafftools.RhythmicStaff("c'8 d'8 e'8 f'8")

    ::

        >>> f(staff)
        \new RhythmicStaff {
            c'8
            d'8
            e'8
            f'8
        }

    ::

        >>> show(staff) # doctest: +SKIP

    Return RhythmicStaff instance.
    '''

    ### CLASS ATTRIBUTES ###

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(self, music=None, context_name='RhythmicStaff', name=None):
        Staff.__init__(self, music=music, context_name=context_name, name=name)



from abjad import *


# TODO: move to test_Measure___init__.py

def test_Measure_in_place_apply_01():

    t = Voice([Note(n, (1, 8)) for n in range(8)])
    leaves_before = t.leaves
    Measure((4, 8), t[0:4])
    leaves_after = t.leaves

    r'''
    \new Voice {
        {
            \time 4/8
            c'8
            cs'8
            d'8
            ef'8
        }
        e'8
        f'8
        fs'8
        g'8
    }
    '''

    assert t.lilypond_format == "\\new Voice {\n\t{\n\t\t\\time 4/8\n\t\tc'8\n\t\tcs'8\n\t\td'8\n\t\tef'8\n\t}\n\te'8\n\tf'8\n\tfs'8\n\tg'8\n}"


def test_Measure_in_place_apply_02():

    t = Staff([Note(n, (1, 8)) for n in range(8)])
    leaves_before = t.leaves
    Measure((4, 8), t[0:4])
    leaves_after = t.leaves

    r'''
    \new Staff {
        {
            \time 4/8
            c'8
            cs'8
            d'8
            ef'8
        }
        e'8
        f'8
        fs'8
        g'8
    }
    '''

    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 4/8\n\t\tc'8\n\t\tcs'8\n\t\td'8\n\t\tef'8\n\t}\n\te'8\n\tf'8\n\tfs'8\n\tg'8\n}"


def test_Measure_in_place_apply_03():

    t = Staff([Note(n, (1, 1)) for n in range(4)])
    leaves_before = t.leaves
    Measure((1, 1), t[0:1])
    leaves_after = t.leaves

    r'''
    \new Staff {
        {
            \time 1/1
            c'1
        }
        cs'1
        d'1
        ef'1
    }
    '''

    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/1\n\t\tc'1\n\t}\n\tcs'1\n\td'1\n\tef'1\n}"


def test_Measure_in_place_apply_04():

    t = Staff([Note(n, (1, 1)) for n in range(4)])
    Measure((1, 1), t[:1])
    Measure((1, 1), t[1:2])
    Measure((1, 1), t[2:3])
    Measure((1, 1), t[3:])
    measuretools.set_always_format_time_signature_of_measures_in_expr(t)

    r'''
    \new Staff {
        {
            \time 1/1
            c'1
        }
        {
            \time 1/1
            cs'1
        }
        {
            \time 1/1
            d'1
        }
        {
            \time 1/1
            ef'1
        }
    }
    '''

    assert wellformednesstools.is_well_formed_component(t)
    assert t.lilypond_format == "\\new Staff {\n\t{\n\t\t\\time 1/1\n\t\tc'1\n\t}\n\t{\n\t\t\\time 1/1\n\t\tcs'1\n\t}\n\t{\n\t\t\\time 1/1\n\t\td'1\n\t}\n\t{\n\t\t\\time 1/1\n\t\tef'1\n\t}\n}"



def rewrite_duration_under_new_tempo(duration, tempo_mark_1, tempo_mark_2):
    r'''.. versionadded:: 2.0

    Rewrite prolated `duration` under new tempo.

    Given prolated `duration` governed by `tempo_mark_1`,
    return new duration governed by `tempo_mark_2`.

    Ensure that `duration` and new duration
    consume exactly the same amount of time in seconds.

    Example. Consider the two tempo indications below. ::

        >>> tempo_mark_1 = contexttools.TempoMark(Duration(1, 4), 60)
        >>> tempo_mark_2 = contexttools.TempoMark(Duration(1, 4), 90)

    The first tempo indication specifies quarter equal to ``60 MM``.

    The second tempo indication specifies quarter equal to ``90 MM``.

    The second tempo is ``3/2`` times as fast as the first::

        >>> tempo_mark_2 / tempo_mark_1
        Multiplier(3, 2)

    Note that a triplet eighth note `tempo_mark_1` equals a regular eighth note
    under `tempo_mark_2`::

        >>> tempotools.rewrite_duration_under_new_tempo(
        ...     Duration(1, 12), tempo_mark_1, tempo_mark_2)
        Duration(1, 8)

    And note that a regular eighth note under `tempo_mark_1` equals a dotted
    sixteenth under `tempo_mark_2`::

        >>> tempotools.rewrite_duration_under_new_tempo(
        ...     Duration(1, 8), tempo_mark_1, tempo_mark_2)
        Duration(3, 16)

    Return duration.
    '''

    tempo_ratio = tempo_mark_2 / tempo_mark_1
    new_duration = tempo_ratio * duration

    return new_duration



from abjad import *


def test_spannertools_destroy_spanners_attached_to_components_in_expr_01():

    staff = Staff("c'4 [ d' ] e' ( f' ) ]")
    assert len(spannertools.get_spanners_attached_to_any_improper_child_of_component(staff)) == 2

    spannertools.destroy_spanners_attached_to_components_in_expr(staff)
    assert not spannertools.get_spanners_attached_to_any_improper_child_of_component(staff)



from abjad.tools.datastructuretools.ObjectInventory import ObjectInventory


class MetricalHierarchyInventory(ObjectInventory):
    '''Abjad model of an ordered list of metrical hierarchies::

        >>> inventory = timesignaturetools.MetricalHierarchyInventory([(4, 4), (3, 4), (6, 8)])

    ::

        >>> z(inventory)
        timesignaturetools.MetricalHierarchyInventory([
            timesignaturetools.MetricalHierarchy(
                '(4/4 (1/4 1/4 1/4 1/4))'
                ),
            timesignaturetools.MetricalHierarchy(
                '(3/4 (1/4 1/4 1/4))'
                ),
            timesignaturetools.MetricalHierarchy(
                '(6/8 ((3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8))))'
                )
            ])

    MetricalHierarchy inventories implement the list interface and are mutable.
    '''

    ### READ-ONLY PRIVATE PROPERTIES ##

    @property
    def _item_callable(self):
        from abjad.tools import timesignaturetools
        return timesignaturetools.MetricalHierarchy




from abjad.tools import sequencetools


def list_inversion_equivalent_chromatic_interval_classes_pairwise(pitch_carriers, wrap=False):
    r'''.. versionadded:: 2.0

    List inversion-equivalent chromatic interval-classes pairwise between `pitch_carriers`::

        >>> staff = Staff("c'8 d'8 e'8 f'8 g'8 a'8 b'8 c''8")

    ::

        >>> f(staff)
        \new Staff {
            c'8
            d'8
            e'8
            f'8
            g'8
            a'8
            b'8
            c''8
        }

    ::

        >>> result = pitchtools.list_inversion_equivalent_chromatic_interval_classes_pairwise(
        ... staff, wrap=False)
        
    ::

        >>> for x in result: x
        ...
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(1)

    ::

        >>> result = pitchtools.list_inversion_equivalent_chromatic_interval_classes_pairwise(
        ... staff, wrap=True)

    ::

        >>> for x in result: x
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(0)

    ::

        >>> notes = staff.leaves
        >>> notes = list(reversed(notes))

    ::

        >>> result = pitchtools.list_inversion_equivalent_chromatic_interval_classes_pairwise(
        ... notes, wrap=False)

    ::

        >>> for x in result: x
        ...
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)

    ::

        >>> result = pitchtools.list_inversion_equivalent_chromatic_interval_classes_pairwise(
        ... notes, wrap=True)

    ::

        >>> for x in result: x
        ...
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(1)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(2)
        InversionEquivalentChromaticIntervalClass(0)

    When ``wrap=False`` do not return ``pitch_carriers[-1] - pitch_carriers[0]`` as last in series.

    When ``wrap=True`` do return ``pitch_carriers[-1] - pitch_carriers[0]`` as last in series.

    Return list.
    '''
    from abjad.tools import pitchtools

    result = []

    if len(pitch_carriers) == 0:
        return result
    elif len(pitch_carriers) == 1:
        if pitchtools.is_pitch_carrier(pitch_carriers[0]):
            return result
        else:
            raise TypeError('must be Abjad pitch, note, note head or chord.')

    if wrap:
        pairs = sequencetools.iterate_sequence_pairwise_wrapped(pitch_carriers)
    else:
        pairs = sequencetools.iterate_sequence_pairwise_strict(pitch_carriers)

    for first_carrier, second_carrier in pairs:
        first_pitch = pitchtools.get_named_chromatic_pitch_from_pitch_carrier(first_carrier)
        second_pitch = pitchtools.get_named_chromatic_pitch_from_pitch_carrier(second_carrier)
        mdi = second_pitch - first_pitch
        iecic = mdi.inversion_equivalent_chromatic_interval_class
        result.append(iecic)

    return result



from abjad import *


def test_Markup_storage_format_01():

    markup = markuptools.Markup(r'\bold { foo }')

    assert repr(markup) == "Markup((MarkupCommand('bold', ['foo']),))"

    r'''
    markuptools.Markup((
        markuptools.MarkupCommand(
            'bold',
            [
                'foo'
            ]
            ),
        ))
    '''

    assert markup.storage_format == "markuptools.Markup((\n\tmarkuptools.MarkupCommand(\n\t\t'bold',\n\t\t[\n\t\t\t'foo'\n\t\t]\n\t\t),\n\t))"


def test_Markup_storage_format_02():

    markup = markuptools.Markup(r'\bold { allegro ma non troppo }', markup_name='non troppo')

    assert repr(markup) == "Markup((MarkupCommand('bold', ['allegro', 'ma', 'non', 'troppo']),), markup_name='non troppo')"

    r'''
    markuptools.Markup((
        markuptools.MarkupCommand(
            'bold',
            [
                'allegro',
                'ma',
                'non',
                'troppo'
            ]
            ),
        ),
        markup_name='non troppo'
        )
    '''

    assert markup.storage_format == "markuptools.Markup((\n\tmarkuptools.MarkupCommand(\n\t\t'bold',\n\t\t[\n\t\t\t'allegro',\n\t\t\t'ma',\n\t\t\t'non',\n\t\t\t'troppo'\n\t\t]\n\t\t),\n\t),\n\tmarkup_name='non troppo'\n\t)"



import abc
from abjad.tools.instrumenttools._SingleReedInstrument import _SingleReedInstrument


class _Saxophone(_SingleReedInstrument):
    r'''.. versionadded:: 2.6

    Abjad model of the family of saxophones.
    '''
    __metaclass__ = abc.ABCMeta

    def __init__(self, **kwargs):
        _SingleReedInstrument.__init__(self, **kwargs)
        self._default_instrument_name = 'saxophone'
        self._default_performer_names.extend(['saxophonist'])
        self._default_short_instrument_name = 'sax.'



from abjad.tools import spannertools


def get_tie_spanner_attached_to_component(component):
    r'''.. versionadded:: 2.10

    Get the only tie spanner attached to `component`::

        >>> staff = Staff("c'8 ~ c'8 d'4")

    ::

        >>> f(staff)
        \new Staff {
            c'8 ~
            c'8
            d'4
        }

    ::

        >>> tietools.get_tie_spanner_attached_to_component(staff[0])
        TieSpanner(c'8, c'8)

    Return tie spanner.

    Raise missing spanner error when no tie spanner attached to `component`.

    Raise extra spanner error when more than one tie spanner attached to `component`.
    '''
    from abjad.tools import tietools

    return spannertools.get_the_only_spanner_attached_to_component(component, tietools.TieSpanner)



def read_abjad_user_config_file(attribute_name):
    '''Read the content of the config file ``$HOME/.abjad/config.py``.

    Returns a dictionary of var : value entries.

    .. versionchanged:: 2.10
        renamed ``configurationtools.read_user_abjad_config_file()`` to
        ``ABJCFG()``.
    '''
    from abjad import ABJCFG

    return ABJCFG[attribute_name]



def list_secondary_instruments():
    r'''.. versionadded:: 2.5

    List secondary instruments::

        >>> for secondary_instrument in instrumenttools.list_secondary_instruments()[:5]:
        ...     secondary_instrument
        ... 
        <class 'abjad.tools.instrumenttools.AltoFlute.AltoFlute.AltoFlute'>
        <class 'abjad.tools.instrumenttools.AltoTrombone.AltoTrombone.AltoTrombone'>
        <class 'abjad.tools.instrumenttools.BaritoneSaxophone.BaritoneSaxophone.BaritoneSaxophone'>
        <class 'abjad.tools.instrumenttools.BassClarinet.BassClarinet.BassClarinet'>
        <class 'abjad.tools.instrumenttools.BassFlute.BassFlute.BassFlute'>

    Return list
    '''
    from abjad.tools import instrumenttools

    secondary_instruments = []

    for instrument_class in instrumenttools.list_instruments():
        instrument = instrument_class()
        if instrument.is_secondary_instrument:
            secondary_instruments.append(instrument_class)

    return secondary_instruments        
